# -*- coding: UTF-8 -*-

try:
    import unittest2 as unittest
except:
    import unittest

from args import ARGS

from oerplib import tools


class TestTools(unittest.TestCase):

    def test_clean_version_numeric(self):
        version = tools.clean_version('6.1')
        self.assertEqual(version, '6.1')

    def test_clean_version_alphanumeric(self):
        version = tools.clean_version('7.0alpha-20121206-000102')
        self.assertEqual(version, '7.0')

    def test_detect_version(self):
        version = tools.detect_version(
            ARGS.server, ARGS.protocol, ARGS.port)
        self.assertIsInstance(version, str)

    def test_v_numeric(self):
        self.assertEqual(tools.v('7.0'), [7, 0])

    def test_v_alphanumeric(self):
        self.assertEqual(tools.v('7.0alpha'), [7, 0])

    def test_v_cmp(self):
        # [(v1, v2, is_inferior), ...]
        versions = [
            ('7.0', '6.1', False), ('6.1', '7.0', True),
            ('7.0alpha', '6.1', False), ('6.1beta', '7.0', True),
            ('6.1beta', '5.0.16', False), ('5.0.16alpha', '6.1', True),
            ('8.0dev-20131102-000101', '7.0-20131014-231047', False),
        ]
        for v1, v2, is_inferior in versions:
            result = tools.v(v1) < tools.v(v2)
            if is_inferior:
                self.assertTrue(result)
            else:
                self.assertFalse(result)

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:



# -*- coding: UTF-8 -*-
##############################################################################
#
#    OERPLib
#    Copyright (C) 2011-2013 Sébastien Alix.
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published
#    by the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU Lesser General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
"""This module provides the BrowseRecord class."""
from oerplib import error


class BrowseRecord(object):
    """Base class that all browsable records inherit from.
    No attributes should be defined in this class (except ``_id``/``id``,
    ``__oerp__``, ``__osv__``, ``__data__`` and Python magic methods) in order
    to not be conflicted with the fields defined in the model class on
    the server.

    A reference to the :class:`OERP <oerplib.OERP>` object used to instanciate
    a ``browse_record`` is available through the ``__oerp__`` attribute::

        >>> oerp = oerplib.OERP('localhost')
        >>> user = oerp.login('admin', 'admin', 'db_name')
        >>> user.__oerp__ == oerp
        True

    The ``__data__`` attribute is used to store some data related to the
    record (it is not recommended to edit them)::

        >>> user.__data__
        {'updated_values': {},
         'raw_data': {'action_id': False,
                      'active': True,
                      'company_id': [1, 'Your Company'],
                      ...},
         'values': {'action_id': False,
                    'active': True,
                    'company_id': [1, 'Your Company'],
                    ...}}

    In the same way, information about the model class and its columns may be
    obtained via the ``__osv__`` attribute::

        >>> user.__osv__
        {'columns': {'action_id': <oerplib.service.osv.fields.Many2OneField object at 0xb75786ec>,
                     'active': <oerplib.service.osv.fields.ValueField object at 0xb7598b6c>,
                     'company_id': <oerplib.service.osv.fields.Many2OneField object at 0xb757868c>,
                     ...},
         'name': 'res.users'}

    """
    __oerp__ = None
    __osv__ = None

    def __init__(self, o_id):
        self._id = o_id
        self.__data__ = {'values': {}, 'raw_data': {}, 'updated_values': {}}

    @property
    def id(self):
        """ID of the record."""
        return self._id

    def __repr__(self):
        return "browse_record(%r, %r)" % (self.__osv__['name'], self._id)

    def __getitem__(self, key):
        return getattr(self, key)

    def __int__(self):
        return self._id

    def __eq__(self, other):
        """Compare two browse records. Return ``True`` if their ID and model
        name are equals.

        NOTE: the comparison is made this way because their model classes can be
        differents objects.

        """
        return isinstance(other, BrowseRecord) and \
            self.id == other.id and \
            self.__osv__['name'] == other.__osv__['name']

    def __ne__(self, other):
        if not isinstance(other, BrowseRecord):
            return True
        return isinstance(other, BrowseRecord)\
            and (self.__osv__['name'], self.id) !=\
                (other.__osv__['name'], other.id)


class BrowseRecordIterator(object):
    """Iterator of browsable records.
    In fact, it is a generator to return records one by one, and able to
    increment/decrement records by overriding '+=' and '-=' operators.
    """
    def __init__(self, model, ids, context=None,
                 parent=None, parent_field=None):
        self.model = model
        self.ids = ids
        self.context = context
        self.index = None
        if self.ids:
            self.index = 0
        self.parent = parent
        self.parent_field = parent_field

    def __len__(self):
        return len(self.ids)

    def __iter__(self):
        return self

    def __next__(self):
        if self.index is None or self.index >= len(self.ids):
            raise StopIteration
        else:
            id_ = self.ids[self.index]
            self.index += 1
            return self.model.browse(id_, context=self.context)

    def next(self):
        self.__next__

    def __iadd__(self, records):
        if not self.parent or not self.parent_field:
            raise error.InternalError("No parent record to update")
        try:
            list(records)
        except TypeError:
            records = [records]
        updated_values = self.parent.__data__['updated_values']
        res = []
        if updated_values.get(self.parent_field.name):
            res = updated_values[self.parent_field.name][:]  # Copy
        from oerplib.service.osv import fields
        for id_ in fields.records2ids(records):
            if (3, id_) in res:
                res.remove((3, id_))
            if (4, id_) not in res:
                res.append((4, id_))
        return res

    def __isub__(self, records):
        if not self.parent or not self.parent_field:
            raise error.InternalError("No parent record to update")
        try:
            list(records)
        except TypeError:
            records = [records]
        updated_values = self.parent.__data__['updated_values']
        res = []
        if updated_values.get(self.parent_field.name):
            res = updated_values[self.parent_field.name][:]  # Copy
        from oerplib.service.osv import fields
        for id_ in fields.records2ids(records):
            if (4, id_) in res:
                res.remove((4, id_))
            if (3, id_) not in res:
                res.append((3, id_))
        return res

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:



# -*- coding: UTF-8 -*-
##############################################################################
#
#    OERPLib
#    Copyright (C) 2011-2013 Sébastien Alix.
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published
#    by the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU Lesser General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

try:
    import xmlrpclib
except ImportError:
    import xmlrpc.client as xmlrpclib

from oerplib.rpc import netrpclib, xmlrpclib_custom, error


class ServiceXMLRPC(object):
    def __init__(self, connector, name, url):
        self._connector = connector
        self._name = name
        self._url = url

    def __getattr__(self, method):
        def rpc_method(*args):
            try:
                self._sock = xmlrpclib_custom.TimeoutServerProxy(
                    self._url, allow_none=True,
                    timeout=self._connector.timeout)
                sock_method = getattr(self._sock, method, False)
                return sock_method(*args)
            #NOTE: exception raised with these kind of requests:
            #   - execute('fake.model', 'search', [])
            #   - execute('sale.order', 'fake_method')
            except xmlrpclib.Fault as exc:
                # faultCode: error message
                # faultString: Server traceback (following the server version
                # used, a bad request can produce a server traceback, or not).
                raise error.ConnectorError(exc.faultCode, exc.faultString)
            #TODO NEED TEST (when is raised this exception?)
            except xmlrpclib.Error as exc:
                raise error.ConnectorError(' - '.join(exc.args))
        return rpc_method


class ServiceNetRPC(object):
    def __init__(self, connector, name, server, port):
        self._connector = connector
        self._name = name
        self._server = server
        self._port = port

    def __getattr__(self, method):
        def rpc_method(*args):
            try:
                sock = netrpclib.NetRPC(timeout=self._connector.timeout)
                sock.connect(self._server, self._port)
                sock.send((self._name, method, ) + args)
                result = sock.receive()
                sock.disconnect()
                return result
            #NOTE: exception raised with these kind of requests:
            #   - execute('fake.model', 'search', [])
            #   - execute('sale.order', 'fake_method')
            except netrpclib.NetRPCError as exc:
                # faultCode: error message
                # faultString: Server traceback (following the server version
                # used, a bad request can produce a server traceback, or not).
                raise error.ConnectorError(exc.faultCode, exc.faultString)
        return rpc_method

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:



# -*- coding: UTF-8 -*-
##############################################################################
#
#    OERPLib
#    Copyright (C) 2011-2013 Sébastien Alix.
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published
#    by the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU Lesser General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
"""Provide the :class:`Wizard` class in order to access the old-style wizards.
"""

from oerplib import rpc, error


class Wizard(object):
    """.. versionadded:: 0.6

    The `Wizard` class represents the ``/wizard`` RPC service which
    lets you access to the old-style wizards.

    .. note::
        This service have to be used through the :attr:`oerplib.OERP.wizard`
        property.

    >>> import oerplib
    >>> oerp = oerplib.OERP('localhost')
    >>> user = oerp.login('admin', 'passwd', 'database')
    >>> oerp.wizard
    <oerplib.service.wizard.Wizard object at 0xb76266ac>

    .. warning::

        All methods documented below are not strictly implemented in `OERPLib`

        Method calls are purely dynamic, and the following documentation can be
        wrong if the API of `OpenERP` is changed between versions. Anyway, if
        you know the API proposed by the server for the ``/wizard`` RPC
        service, it will work.

    .. method:: Wizard.create(wiz_name, datas=None)

        >>> oerp.wizard.create('wiz_name')
        1

        :return: the wizard's instance ID

    .. method:: Wizard.execute(wiz_id, datas, action='init', context=None)

        >>> oerp.wizard.execute(1, {'one_field': "Value"})

    """
    def __init__(self, oerp):
        self._oerp = oerp

    def __getattr__(self, method):
        """Provide a dynamic access to a RPC method."""
        def rpc_method(*args):
            """Return the result of the RPC request."""
            try:
                meth = getattr(self._oerp._connector.wizard, method, False)
                return meth(self._oerp.database, self._oerp.user.id, *args)
            except rpc.error.ConnectorError as exc:
                raise error.RPCError(exc.message, exc.oerp_traceback)
        return rpc_method

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:



# -*- coding: UTF-8 -*-
##############################################################################
#
#    OERPLib
#    Copyright (C) 2011-2013 Sébastien Alix.
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published
#    by the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU Lesser General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
"""The `oerplib` module defines the :class:`OERP` class.

The :class:`OERP` class is the entry point to manage `OpenERP/Odoo` servers.
You can use this one to write `Python` programs that
performs a variety of automated jobs that communicate with a
`OpenERP/Odoo` server.

You can load a pre-configured :class:`OERP` session with the :func:`load`
function.
"""

__author__ = 'ABF Osiell - Sebastien Alix'
__email__ = 'sebastien.alix@osiell.com'
__licence__ = 'LGPL v3'
__version__ = '0.8.6'

#__all__ = ['OERP', 'error']

from oerplib.oerp import OERP
from oerplib import error

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:



# -*- coding: UTF-8 -*-

try:
    import unittest2 as unittest
except:
    import unittest
import tempfile
import os

from args import ARGS

import oerplib


class TestSession(unittest.TestCase):

    def setUp(self):
        self.oerp = oerplib.OERP(
            ARGS.server, protocol=ARGS.protocol, port=ARGS.port,
            version=ARGS.version)
        self.user = self.oerp.login(ARGS.user, ARGS.passwd, ARGS.database)
        self.session_name = ARGS.database
        self.file_path = tempfile.mkstemp(suffix='.cfg', prefix='oerplib_')[1]

    def tearDown(self):
        os.remove(self.file_path)

    def test_session_oerp_list(self):
        result = oerplib.OERP.list(rc_file=self.file_path)
        self.assertIsInstance(result, list)
        other_file_path = tempfile.mkstemp()[1]
        result = oerplib.OERP.list(rc_file=other_file_path)
        self.assertIsInstance(result, list)

    def test_session_oerp_save_and_remove(self):
        self.oerp.save(self.session_name, rc_file=self.file_path)
        result = oerplib.OERP.list(rc_file=self.file_path)
        self.assertIn(self.session_name, result)
        oerplib.OERP.remove(self.session_name, rc_file=self.file_path)

    def test_session_oerp_load(self):
        self.oerp.save(self.session_name, rc_file=self.file_path)
        oerp = oerplib.OERP.load(self.session_name, rc_file=self.file_path)
        self.assertIsInstance(oerp, oerplib.OERP)
        self.assertEqual(self.oerp.server, oerp.server)
        self.assertEqual(self.oerp.port, oerp.port)
        self.assertEqual(self.oerp.database, oerp.database)
        self.assertEqual(self.oerp.protocol, oerp.protocol)
        self.assertEqual(self.oerp.user, oerp.user)
        oerplib.OERP.remove(self.session_name, rc_file=self.file_path)

    def test_session_tools_get(self):
        self.oerp.save(self.session_name, rc_file=self.file_path)
        data = {
            'type': self.oerp.__class__.__name__,
            'server': self.oerp.server,
            'protocol': self.oerp.protocol,
            'port': int(self.oerp.port),
            'timeout': self.oerp.config['timeout'],
            'user': self.oerp.user.login,
            'passwd': self.oerp._password,
            'database': self.oerp.database,
        }
        result = oerplib.tools.session.get(
            self.session_name, rc_file=self.file_path)
        self.assertEqual(data, result)
        oerplib.OERP.remove(self.session_name, rc_file=self.file_path)

    def test_session_tools_get_all(self):
        self.oerp.save(self.session_name, rc_file=self.file_path)
        data = {
            self.session_name: {
                'type': self.oerp.__class__.__name__,
                'server': self.oerp.server,
                'protocol': self.oerp.protocol,
                'port': int(self.oerp.port),
                'timeout': self.oerp.config['timeout'],
                'user': self.oerp.user.login,
                'passwd': self.oerp._password,
                'database': self.oerp.database,
            }
        }
        result = oerplib.tools.session.get_all(rc_file=self.file_path)
        self.assertIn(self.session_name, result)
        self.assertEqual(data, result)
        oerplib.OERP.remove(self.session_name, rc_file=self.file_path)

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:



# -*- coding: UTF-8 -*-
##############################################################################
#
#    OERPLib
#    Copyright (C) 2011-2013 Sébastien Alix.
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published
#    by the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU Lesser General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
"""This module contains classes representing the fields supported by OpenObject.
A field is a Python descriptor which defines getter/setter methods for
its related attribute.
"""
import datetime

from oerplib import error
from oerplib.service.osv import browse

try:
    basestring
except:
    basestring = str

def is_int(value):
    """Return `True` if ``value`` is an integer."""
    if isinstance(value, bool):
        return False
    try:
        int(value)
        return True
    except ValueError:
        return False


def oerp_tuple_in(iterable):
    """Return `True` if `iterable` contains an expected tuple like
    ``(6, 0, IDS)`` (and so on).

        >>> oerp_tuple_in([0, 1, 2])        # Simple list
        False
        >>> oerp_tuple_in([(6, 0, [42])])   # List of tuples
        True
        >>> oerp_tuple_in([[1, 42]])        # List of lists
        True
    """
    if not iterable:
        return False
    def is_oerp_tuple(elt):
        try:
            return elt[:1][0] in [1, 2, 3, 4, 5] \
                    or elt[:2] in [(6, 0), [6, 0], (0, 0), [0, 0]]
        except:
            return False
    return any(is_oerp_tuple(elt) for elt in iterable)


def records2ids(iterable):
    """Replace `browse_records` contained in `iterable` by their
    corresponding IDs:

        >>> groups = list(oerp.user.groups_id)
        >>> records2ids(groups)
        [1, 2, 3, 14, 17, 18, 19, 7, 8, 9, 5, 20, 21, 22, 23]
    """
    def record2id(elt):
        if isinstance(elt, browse.BrowseRecord):
            return elt.id
        return elt
    return [record2id(elt) for elt in iterable]


class BaseField(object):
    """Field which all other fields inherit.
    Manage common metadata.
    """
    def __init__(self, osv, name, data):
        self.osv = osv
        self.name = name
        self.type = 'type' in data and data['type'] or False
        self.string = 'string' in data and data['string'] or False
        self.size = 'size' in data and data['size'] or False
        self.required = 'required' in data and data['required'] or False
        self.readonly = 'readonly' in data and data['readonly'] or False
        self.help = 'help' in data and data['help'] or False
        self.states = 'states' in data and data['states'] or False

    def __get__(self, instance, owner):
        pass

    def __set__(self, instance, value):
        pass

    def __str__(self):
        """Return a human readable string representation of the field."""
        attrs = ['string', 'relation', 'required', 'readonly', 'size', 'domain']
        attrs_rep = []
        for attr in attrs:
            if hasattr(self, attr):
                value = getattr(self, attr)
                if value:
                    if isinstance(value, basestring):
                        attrs_rep.append("{0}='{1}'".format(attr, value))
                    else:
                        attrs_rep.append("{0}={1}".format(attr, value))
        attrs_rep = ", ".join(attrs_rep)
        return "{0}({1})".format(self.type, attrs_rep)

    def check_value(self, value):
        """Check the validity of a value for the field."""
        #if self.readonly:
        #    raise error.Error(
        #        "'{field_name}' field is readonly".format(
        #            field_name=self.name))
        if value and self.size:
            if not isinstance(value, basestring):
                raise ValueError("Value supplied has to be a string")
            if len(value) > self.size:
                raise ValueError(
                    "Lenght of the '{field_name}' is limited to {size}".format(
                        field_name=self.name,
                        size=self.size))
        if not value and self.required:
            raise ValueError(
                "'{field_name}' field is required".format(
                    field_name=self.name))
        return value


class SelectionField(BaseField):
    """Represent the OpenObject 'fields.selection'"""
    def __init__(self, osv, name, data):
        super(SelectionField, self).__init__(osv, name, data)
        self.selection = 'selection' in data and data['selection'] or False

    def __get__(self, instance, owner):
        if self.name in instance.__data__['updated_values']:
            return instance.__data__['updated_values'][self.name]
        return instance.__data__['values'][self.name]

    def __set__(self, instance, value):
        value = self.check_value(value)
        instance.__data__['updated_values'][self.name] = value

    def check_value(self, value):
        super(SelectionField, self).check_value(value)
        selection = [val[0] for val in self.selection]
        if value and value not in selection:
            raise ValueError(
                "The value '{value}' supplied doesn't match with the possible "
                "values '{selection}' for the '{field_name}' field".format(
                    value=value,
                    selection=selection,
                    field_name=self.name,
                ))
        return value


class Many2ManyField(BaseField):
    """Represent the OpenObject 'fields.many2many'"""
    def __init__(self, osv, name, data):
        super(Many2ManyField, self).__init__(osv, name, data)
        self.relation = 'relation' in data and data['relation'] or False
        self.context = 'context' in data and data['context'] or {}
        self.domain = 'domain' in data and data['domain'] or False

    def __get__(self, instance, owner):
        """Return a generator to iterate on ``browse_record`` instances."""
        ids = None
        if instance.__data__['values'][self.name]:
            ids = instance.__data__['values'][self.name][:]
        # None value => get the value on the fly
        if ids is None:
            orig_ids = instance.__oerp__.read(
                instance.__osv__['name'],
                [instance.id], [self.name])[0][self.name]
            instance.__data__['values'][self.name] = orig_ids
            ids = orig_ids and orig_ids[:] or []
        # Take updated values into account
        if self.name in instance.__data__['updated_values']:
            ids = ids or []
            values = instance.__data__['updated_values'][self.name]
            # Handle OERP tuples to update 'ids'
            for value in values:
                if value[0] == 6 and value[2]:
                    ids = value[2]
                elif value[0] == 5:
                    ids = []
                elif value[0] == 4 and value[1] and value[1] not in ids:
                    ids.append(value[1])
                elif value[0] == 3 and value[1] and value[1] in ids:
                    ids.remove(value[1])
        context = instance.__data__['context'].copy()
        context.update(self.context)
        return browse.BrowseRecordIterator(
            model=instance.__oerp__.get(self.relation),
            ids=ids,
            context=context,
            parent=instance,
            parent_field=self)

    def __set__(self, instance, value):
        value = self.check_value(value)
        if value and not oerp_tuple_in(value):
            value = [(6, 0, records2ids(value))]
        elif not value:
            value = [(5, )]
        instance.__data__['updated_values'][self.name] = value

    def check_value(self, value):
        if value:
            if not isinstance(value, list):
                raise ValueError(
                    "The value supplied has to be a list or 'False'")
        return super(Many2ManyField, self).check_value(value)


class Many2OneField(BaseField):
    """Represent the OpenObject 'fields.many2one'"""
    def __init__(self, osv, name, data):
        super(Many2OneField, self).__init__(osv, name, data)
        self.relation = 'relation' in data and data['relation'] or False
        self.context = 'context' in data and data['context'] or {}
        self.domain = 'domain' in data and data['domain'] or False

    def __get__(self, instance, owner):
        id_ = instance.__data__['values'][self.name]
        if self.name in instance.__data__['updated_values']:
            id_ = instance.__data__['updated_values'][self.name]
            # FIXME if id_ is a browse_record
        # None value => get the value on the fly
        if id_ is None:
            id_ = instance.__oerp__.read(
                instance.__osv__['name'],
                [instance.id], [self.name])[0][self.name]
            instance.__data__['values'][self.name] = id_
        if id_:
            context = instance.__data__['context'].copy()
            context.update(self.context)
            return instance.__class__.__oerp__.browse(
                self.relation, id_[0], context)
        return False

    def __set__(self, instance, value):
        if isinstance(value, browse.BrowseRecord):
            o_rel = value
        elif is_int(value):
            o_rel = instance.__class__.__oerp__.browse(self.relation, value)
        elif value in [None, False]:
            o_rel = False
        else:
            raise ValueError("Value supplied has to be an integer, "
                             "a browse_record object or False.")
        o_rel = self.check_value(o_rel)
        instance.__data__['updated_values'][self.name] = \
            o_rel and [o_rel.id, False]

    def check_value(self, value):
        super(Many2OneField, self).check_value(value)
        if value and value.__osv__['name'] != self.relation:
            raise ValueError(
                ("Instance of '{model}' supplied doesn't match with the " +
                 "relation '{relation}' of the '{field_name}' field.").format(
                     model=value.__osv__['name'],
                     relation=self.relation,
                     field_name=self.name))
        return value


class One2ManyField(BaseField):
    """Represent the OpenObject 'fields.one2many'"""
    def __init__(self, osv, name, data):
        super(One2ManyField, self).__init__(osv, name, data)
        self.relation = 'relation' in data and data['relation'] or False
        self.context = 'context' in data and data['context'] or {}
        self.domain = 'domain' in data and data['domain'] or False

    def __get__(self, instance, owner):
        """Return a generator to iterate on ``browse_record`` instances."""
        ids = None
        if instance.__data__['values'][self.name]:
            ids = instance.__data__['values'][self.name][:]
        # None value => get the value on the fly
        if ids is None:
            orig_ids = instance.__oerp__.read(
                instance.__osv__['name'],
                [instance.id], [self.name])[0][self.name]
            instance.__data__['values'][self.name] = orig_ids
            ids = orig_ids and orig_ids[:] or []
        # Take updated values into account
        if self.name in instance.__data__['updated_values']:
            ids = ids or []
            values = instance.__data__['updated_values'][self.name]
            # Handle OERP tuples to update 'ids'
            for value in values:
                if value[0] == 6 and value[2]:
                    ids = value[2]
                elif value[0] == 5:
                    ids = []
                elif value[0] == 4 and value[1] and value[1] not in ids:
                    ids.append(value[1])
                elif value[0] == 3 and value[1] and value[1] in ids:
                    ids.remove(value[1])
        context = instance.__data__['context'].copy()
        context.update(self.context)
        return browse.BrowseRecordIterator(
            model=instance.__oerp__.get(self.relation),
            ids=ids,
            context=context,
            parent=instance,
            parent_field=self)

    def __set__(self, instance, value):
        value = self.check_value(value)
        if value and not oerp_tuple_in(value):
            value = [(6, 0, records2ids(value))]
        elif not value:
            value = [(5, )]
        instance.__data__['updated_values'][self.name] = value

    def check_value(self, value):
        if value:
            if not isinstance(value, list):
                raise ValueError(
                    "The value supplied has to be a list or 'False'")
        return super(One2ManyField, self).check_value(value)


class ReferenceField(BaseField):
    """.. versionadded:: 0.6
    Represent the OpenObject 'fields.reference'.
    """
    def __init__(self, osv, name, data):
        super(ReferenceField, self).__init__(osv, name, data)
        self.context = 'context' in data and data['context'] or {}
        self.domain = 'domain' in data and data['domain'] or False
        self.selection = 'selection' in data and data['selection'] or False

    def __get__(self, instance, owner):
        value = instance.__data__['values'][self.name]
        if self.name in instance.__data__['updated_values']:
            value = instance.__data__['updated_values'][self.name]
        # None value => get the value on the fly
        if value is None:
            value = instance.__oerp__.read(
                instance.__osv__['name'],
                [instance.id], [self.name])[0][self.name]
            instance.__data__['values'][self.name] = value
        if value:
            relation, sep, o_id = value.rpartition(',')
            relation = relation.strip()
            o_id = int(o_id.strip())
            if relation and o_id:
                context = instance.__data__['context'].copy()
                context.update(self.context)
                return instance.__class__.__oerp__.browse(
                    relation, o_id, context)
        return False

    def __set__(self, instance, value):
        value = self.check_value(value)
        instance.__data__['updated_values'][self.name] = value

    def _check_relation(self, relation):
        selection = [val[0] for val in self.selection]
        if relation not in selection:
            raise ValueError(
                ("The value '{value}' supplied doesn't match with the possible"
                 " values '{selection}' for the '{field_name}' field").format(
                     value=relation,
                     selection=selection,
                     field_name=self.name,
                 ))
        return relation

    def check_value(self, value):
        if isinstance(value, browse.BrowseRecord):
            relation = value.__class__.__osv__['name']
            self._check_relation(relation)
            value = "%s,%s" % (relation, value.id)
            super(ReferenceField, self).check_value(value)
        elif isinstance(value, basestring):
            super(ReferenceField, self).check_value(value)
            relation, sep, o_id = value.rpartition(',')
            relation = relation.strip()
            o_id = o_id.strip()
            #o_rel = instance.__class__.__oerp__.browse(relation, o_id)
            if not relation or not is_int(o_id):
                raise ValueError("String not well formatted, expecting "
                                 "'{relation},{id}' format")
            self._check_relation(relation)
        else:
            raise ValueError("Value supplied has to be a string or"
                             " a browse_record object.")
        return value


class DateField(BaseField):
    """Represent the OpenObject 'fields.data'"""
    pattern = "%Y-%m-%d"

    def __init__(self, osv, name, data):
        super(DateField, self).__init__(osv, name, data)

    def __get__(self, instance, owner):
        value = instance.__data__['values'][self.name]
        if self.name in instance.__data__['updated_values']:
            value = instance.__data__['updated_values'][self.name]
        try:
            res = datetime.datetime.strptime(value, self.pattern).date()
        except Exception:  # ValueError, TypeError
            res = value
        return res

    def __set__(self, instance, value):
        value = self.check_value(value)
        instance.__data__['updated_values'][self.name] = value

    def check_value(self, value):
        super(DateField, self).check_value(value)
        if isinstance(value, datetime.date):
            value = value.strftime("%Y-%m-%d")
        elif isinstance(value, basestring):
            try:
                datetime.datetime.strptime(value, self.pattern)
            except:
                raise ValueError(
                    "String not well formatted, expecting '{0}' format".format(
                        self.pattern))
        elif isinstance(value, bool):
            return value
        else:
            raise ValueError(
                "Expecting a datetime.date object or basestring")
        return value


class DateTimeField(BaseField):
    """Represent the OpenObject 'fields.datetime'"""
    pattern = "%Y-%m-%d %H:%M:%S"

    def __init__(self, osv, name, data):
        super(DateTimeField, self).__init__(osv, name, data)

    def __get__(self, instance, owner):
        value = instance.__data__['values'][self.name]
        if self.name in instance.__data__['updated_values']:
            value = instance.__data__['updated_values'][self.name]
        try:
            res = datetime.datetime.strptime(value, self.pattern)
        except Exception:  # ValueError, TypeError
            res = value
        return res

    def __set__(self, instance, value):
        value = self.check_value(value)
        instance.__data__['updated_values'][self.name] = value

    def check_value(self, value):
        super(DateTimeField, self).check_value(value)
        if isinstance(value, datetime.datetime):
            value = value.strftime("%Y-%m-%d %H:%M:%S")
        elif isinstance(value, basestring):
            try:
                datetime.datetime.strptime(value, self.pattern)
            except:
                raise ValueError(
                    "Value not well formatted, expecting '{0}' format".format(
                        self.pattern))
        elif isinstance(value, bool):
            return value
        else:
            raise ValueError(
                "Expecting a datetime.datetime object or basestring")
        return value


class ValueField(BaseField):
    """Represent simple OpenObject fields:
    - 'fields.char',
    - 'fields.float',
    - 'fields.integer',
    - 'fields.boolean',
    - 'fields.text',
    - 'fields.binary',
    """
    def __init__(self, osv, name, data):
        super(ValueField, self).__init__(osv, name, data)

    def __get__(self, instance, owner):
        if self.name in instance.__data__['updated_values']:
            return instance.__data__['updated_values'][self.name]
        return instance.__data__['values'][self.name]

    def __set__(self, instance, value):
        value = self.check_value(value)
        instance.__data__['updated_values'][self.name] = value


def generate_field(osv, name, data):
    """Generate a well-typed field according to the data dictionary supplied
    (obtained via 'fields_get' XML-RPC/NET-RPC method).

    """
    assert 'type' in data
    field = None
    if data['type'] == 'selection':
        field = SelectionField(osv, name, data)
    elif data['type'] == 'many2many':
        field = Many2ManyField(osv, name, data)
    elif data['type'] == 'many2one':
        field = Many2OneField(osv, name, data)
    elif data['type'] == 'one2many':
        field = One2ManyField(osv, name, data)
    elif data['type'] == 'reference':
        field = ReferenceField(osv, name, data)
    elif data['type'] == 'date':
        field = DateField(osv, name, data)
    elif data['type'] == 'datetime':
        field = DateTimeField(osv, name, data)
    elif data['type'] in ['char', 'float', 'integer', 'integer_big',
                          'boolean', 'text', 'binary', 'html']:
        field = ValueField(osv, name, data)
    else:
        field = ValueField(osv, name, data)
    return field

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:



# -*- coding: UTF-8 -*-
##############################################################################
#
#    OERPLib
#    Copyright (C) 2011-2013 Sébastien Alix.
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published
#    by the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU Lesser General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
"""This module provides `RPC` connectors which use the `XML-RPC`, `Net-RPC`
or `JSON-RPC` protocol to communicate with an `OpenERP/Odoo` server.

Afterwards, `RPC` services and their associated methods can be accessed
dynamically from the connector returned.

`XML-RPC` and `Net-RPC` provide the same interface, such as services like
``db``, ``common`` or ``object``.
On the other hand, `JSON-RPC` provides a completely different interface, with
services provided by Web modules like ``web/session``,
``web/dataset`` and so on.
"""
from oerplib.rpc import error, service, jsonrpclib
from oerplib.tools import v

# XML-RPC available URL
# '/xmlrpc'             => 5.0, 6.0, 6.1, 7.0, 8.0 (legacy path)
# '/openerp/xmlrpc/1'   => 6.1, 7.0
# '/xmlrpc/2'           => 8.0
XML_RPC_PATHS = ['/xmlrpc', '/openerp/xmlrpc/1', '/xmlrpc/2']


class Connector(object):
    """Connector base class defining the interface used
    to interact with a server.
    """
    def __init__(self, server, port=8069, timeout=120, version=None):
        self.server = server
        try:
            int(port)
        except ValueError:
            txt = "The port '{0}' is invalid. An integer is required."
            txt = txt.format(port)
            raise error.ConnectorError(txt)
        else:
            self.port = int(port)
        self._timeout = timeout
        self.version = version
        self._url = None

    @property
    def timeout(self):
        return self._timeout

    @timeout.setter
    def timeout(self, timeout):
        self._timeout = timeout


class ConnectorXMLRPC(Connector):
    """Connector class using the `XML-RPC` protocol.

    >>> from oerplib import rpc
    >>> cnt = rpc.ConnectorXMLRPC('localhost', port=8069)

    Login and retrieve ID of the user connected:

    >>> uid = cnt.common.login('database', 'user', 'passwd')

    Execute a query:

    >>> res = cnt.object.execute('database', uid, 'passwd', 'res.partner', 'read', [1])

    Execute a workflow query:

    >>> res = cnt.object.exec_workflow('database', uid, 'passwd', 'sale.order', 'order_confirm', 4)
    """
    def __init__(self, server, port=8069, timeout=120, version=None, scheme='http'):
        super(ConnectorXMLRPC, self).__init__(server, port, timeout, version)
        self.scheme = scheme
        if self.version:
            # Server < 6.1
            if v(self.version) < v('6.1'):
                self._url = '{scheme}://{server}:{port}/xmlrpc'.format(
                    scheme=self.scheme, server=self.server, port=self.port)
            # Server >= 6.1 and < 8.0
            elif v(self.version) < v('8.0'):
                self._url = '{scheme}://{server}:{port}/openerp/xmlrpc/1'.format(
                    scheme=self.scheme, server=self.server, port=self.port)
            # Server >= 8.0
            elif v(self.version) >= v('8.0'):
                self._url = '{scheme}://{server}:{port}/xmlrpc/2'.format(
                    scheme=self.scheme, server=self.server, port=self.port)
        # Detect the XML-RPC path to use
        if self._url is None:
            # We begin with the last known XML-RPC path to give the priority to
            # the last server version supported
            paths = XML_RPC_PATHS[:]
            paths.reverse()
            for path in paths:
                url = '{scheme}://{server}:{port}{path}'.format(
                    scheme=self.scheme, server=self.server, port=self.port, path=path)
                try:
                    db = service.ServiceXMLRPC(
                        self, 'db', '{url}/{srv}'.format(url=url, srv='db'))
                    version = db.server_version()
                except error.ConnectorError:
                    continue
                else:
                    self._url = url
                    self.version = version
                    break

    def __getattr__(self, service_name):
        url = self._url + '/' + service_name
        srv = service.ServiceXMLRPC(self, service_name, url)
        setattr(self, service_name, srv)
        return srv


class ConnectorXMLRPCSSL(ConnectorXMLRPC):
    """Connector class using the `XML-RPC` protocol over `SSL`."""
    def __init__(self, server, port=8069, timeout=120, version=None):
        super(ConnectorXMLRPCSSL, self).__init__(
            server, port, timeout, version, scheme='https')

class ConnectorNetRPC(Connector):
    """
    .. note::
        No longer available since `OpenERP 7.0`.

    Connector class using the `Net-RPC` protocol.
    """
    def __init__(self, server, port=8070, timeout=120, version=None):
        super(ConnectorNetRPC, self).__init__(
            server, port, timeout, version)
        if self.version is None:
            try:
                db = service.ServiceNetRPC(self, 'db', self.server, self.port)
                version = db.server_version()
            except error.ConnectorError:
                pass
            else:
                self.version = version

    def __getattr__(self, service_name):
        srv = service.ServiceNetRPC(
            self, service_name, self.server, self.port)
        setattr(self, service_name, srv)
        return srv


class ConnectorJSONRPC(Connector):
    """Connector class using the `JSON-RPC` protocol.

    >>> from oerplib import rpc
    >>> cnt = rpc.ConnectorJSONRPC('localhost', port=8069)

    Open a user session:

    >>> cnt.proxy.web.session.authenticate(db='database', login='admin', password='admin')
    {u'jsonrpc': u'2.0', u'id': 202516757,
     u'result': {u'username': u'admin', u'user_context': {u'lang': u'fr_FR', u'tz': u'Europe/Brussels', u'uid': 1},
     u'db': u'test70', u'uid': 1, u'session_id': u'308816f081394a9c803613895b988540'}}

    Read data of a partner:

    >>> cnt.proxy.web.dataset.call(model='res.partner', method='read', args=[[1]])
    {u'jsonrpc': u'2.0', u'id': 454236230,
     u'result': [{u'id': 1, u'comment': False, u'ean13': False, u'property_account_position': False, ...}]}

    You can send requests this way too:

    >>> cnt.proxy['/web/dataset'].call(model='res.partner', method='read', args=[[1]])
    {u'jsonrpc': u'2.0', u'id': 328686288,
     u'result': [{u'id': 1, u'comment': False, u'ean13': False, u'property_account_position': False, ...}]}

    Or like this:

    >>> cnt.proxy['web']['dataset'].call(model='res.partner', method='read', args=[[1]])
    {u'jsonrpc': u'2.0', u'id': 102320639,
     u'result': [{u'id': 1, u'comment': False, u'ean13': False, u'property_account_position': False, ...}]}
    """
    def __init__(self, server, port=8069, timeout=120, version=None,
                 deserialize=True):
        super(ConnectorJSONRPC, self).__init__(server, port, timeout, version)
        self.deserialize = deserialize
        self._proxy = self._get_proxy(ssl=False)

    def _get_proxy(self, ssl=False):
        """Returns a :class:`Proxy <oerplib.rpc.jsonrpclib.Proxy>` instance
        corresponding to the server version used.
        """
        # Detect the server version
        if self.version is None:
            proxy = jsonrpclib.Proxy(
                self.server, self.port, self._timeout,
                ssl=ssl, deserialize=self.deserialize)
            result = proxy.web.webclient.version_info()['result']
            # Server 6.1
            if 'version' in result:
                self.version = result['version']
            # Server >= 7.0
            elif 'server_version' in result:
                self.version = result['server_version']
        # Select the legacy proxy for OpenERP 6.1 and 7.0
        if v(self.version)[:2] <= v('7.0'):
            proxy = jsonrpclib.ProxyLegacy(
                self.server, self.port, self._timeout,
                ssl=ssl, deserialize=self.deserialize)
        return proxy

    @property
    def proxy(self):
        return self._proxy

    @property
    def timeout(self):
        return self._proxy._timeout

    @timeout.setter
    def timeout(self, timeout):
        self._proxy._timeout = timeout


class ConnectorJSONRPCSSL(ConnectorJSONRPC):
    """Connector class using the `JSON-RPC` protocol over `SSL`.

    >>> from oerplib import rpc
    >>> cnt = rpc.ConnectorJSONRPCSSL('localhost', port=8069)
    """
    def __init__(self, server, port=8069, timeout=120, version=None,
                 deserialize=True):
        super(ConnectorJSONRPCSSL, self).__init__(
            server, port, timeout, version)
        self._proxy = self._get_proxy(ssl=True)


PROTOCOLS = {
    'xmlrpc': ConnectorXMLRPC,
    'xmlrpc+ssl': ConnectorXMLRPCSSL,
    #'jsonrpc': ConnectorJSONRPC,
    #'jsonrpc+ssl': ConnectorJSONRPCSSL,
    'netrpc': ConnectorNetRPC,
}


def get_connector(server, port=8069, protocol='xmlrpc',
                  timeout=120, version=None):
    """
    .. deprecated:: 0.8
    
    Return a `RPC` connector to interact with an `OpenERP/Odoo` server.
    Supported protocols are:

        - ``xmlrpc``: Standard `XML-RPC` protocol (default),
        - ``xmlrpc+ssl``: `XML-RPC` protocol over `SSL`,
        - ``netrpc``: `Net-RPC` protocol (no longer available
          since `OpenERP 7.0`).

    If the `version` parameter is set to `None`, the last API supported will
    be used to send requests to the server. Otherwise, you can force the
    API to use with the corresponding string version
    (e.g.: ``'6.0', '6.1', '7.0', '8.0', ...``):

        >>> from oerplib import rpc
        >>> cnt = rpc.get_connector('localhost', 8069, 'xmlrpc', version='7.0')
    """
    if protocol not in PROTOCOLS:
        txt = ("The protocol '{0}' is not supported. "
               "Please choose a protocol among these ones: {1}")
        txt = txt.format(protocol, list(PROTOCOLS.keys()))
        raise error.ConnectorError(txt)
    return PROTOCOLS[protocol](server, port, timeout, version)

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:



# -*- coding: UTF-8 -*-

try:
    import unittest2 as unittest
except:
    import unittest

from args import ARGS

import oerplib


class TestInit(unittest.TestCase):

    def test_init1(self):
        # Server + Database + Protocol + Port
        oerp = oerplib.OERP(ARGS.server, ARGS.database,
                            ARGS.protocol, ARGS.port)
        self.assertIsInstance(oerp, oerplib.OERP)
        self.assertIsNotNone(oerp)
        self.assertEqual(oerp.server, ARGS.server)
        self.assertEqual(oerp.database, ARGS.database)
        self.assertEqual(oerp.protocol, ARGS.protocol)
        self.assertEqual(oerp.port, ARGS.port)

    def test_init2(self):
        # Server + Database + Protocol + Port + Timeout
        oerp = oerplib.OERP(ARGS.server, ARGS.database,
                            ARGS.protocol, ARGS.port, 42)
        self.assertIsInstance(oerp, oerplib.OERP)
        self.assertIsNotNone(oerp)
        self.assertEqual(oerp.server, ARGS.server)
        self.assertEqual(oerp.database, ARGS.database)
        self.assertEqual(oerp.protocol, ARGS.protocol)
        self.assertEqual(oerp.port, ARGS.port)
        self.assertEqual(oerp.config['timeout'], 42)

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:



# -*- coding: UTF-8 -*-

try:
    import unittest2 as unittest
except:
    import unittest

from args import ARGS

import oerplib
from oerplib.tools import v


class TestDBDrop(unittest.TestCase):

    def setUp(self):
        self.oerp = oerplib.OERP(
            ARGS.server, protocol=ARGS.protocol, port=ARGS.port,
            version=ARGS.version)

    def test_db_drop_existing_database(self):
        res = self.oerp.db.drop(ARGS.super_admin_passwd, ARGS.database)
        self.assertTrue(res)
        db_list = self.oerp.db.list()
        self.assertNotIn(ARGS.database, db_list)

    def test_db_drop_no_existing_database(self):
        if v(self.oerp.version) >= v('6.1'):
            res = self.oerp.db.drop(ARGS.super_admin_passwd, 'fake_db_name')
            self.assertFalse(res)
        else:
            self.assertRaises(
                oerplib.error.RPCError,
                self.oerp.db.drop,
                ARGS.super_admin_passwd, 'fake_db_name')

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:



#!/usr/bin/env python
# -*- coding: UTF-8 -*-

try:
    import unittest2 as unittest
except:
    import unittest

from args import ARGS

from test_tools import TestTools
from test_init import TestInit
from test_login import TestLogin
from test_db_create import TestDBCreate
from test_db import TestDB
from test_db_drop import TestDBDrop
from test_execute import TestExecute
from test_execute_kw import TestExecuteKw
from test_browse import TestBrowse
from test_osv import TestOSV
from test_timeout import TestTimeout
from test_session import TestSession
from test_inspect import TestInspect

from oerplib.tools import v

def runTest(suite, f):
    # Run all tests
    if ARGS.test_xmlrpc:
        print("-- RUN (XMLRPC) --")
        ARGS.protocol = 'xmlrpc'
        ARGS.port = int(ARGS.xmlrpc_port)
        unittest.TextTestRunner(verbosity=ARGS.verbosity, stream=f).run(suite)
    if ARGS.test_netrpc:
        print("-- RUN (NETRPC) --")
        ARGS.protocol = 'netrpc'
        ARGS.port = int(ARGS.netrpc_port)
        unittest.TextTestRunner(verbosity=ARGS.verbosity, stream=f).run(suite)
    if not ARGS.test_xmlrpc and not ARGS.test_netrpc:
        print("-- NO TEST --")
        print("Please use '--test_xmlrpc' and/or '--test_netrpc' option.")

if __name__ == '__main__':
    suite = unittest.TestSuite()

    #---------------
    #- First Tests -
    #---------------

    # 1) Test oerplib.tools
    loader = unittest.TestLoader().loadTestsFromTestCase(TestTools)
    suite.addTest(loader)

    # 2) Test OERP.__init__
    loader = unittest.TestLoader().loadTestsFromTestCase(TestInit)
    suite.addTest(loader)
    # 3) Test OERP.db (create the database)
    if ARGS.create_db:
        loader = unittest.TestLoader().loadTestsFromTestCase(TestDBCreate)
        suite.addTest(loader)
    else:
        print("-- TestDBCreate skipped --")
    # 4) Test OERP.login
    loader = unittest.TestLoader().loadTestsFromTestCase(TestLogin)
    suite.addTest(loader)

    #---------
    #- Tests -
    #---------

    # Test OERP.db
    loader = unittest.TestLoader().loadTestsFromTestCase(TestDB)
    suite.addTest(loader)

    # Test OERP.execute and OERP.execute_kw
    loader = unittest.TestLoader().loadTestsFromTestCase(TestExecute)
    suite.addTest(loader)
    loader = unittest.TestLoader().loadTestsFromTestCase(TestExecuteKw)
    suite.addTest(loader)

    # Test OERP.browse
    loader = unittest.TestLoader().loadTestsFromTestCase(TestBrowse)
    suite.addTest(loader)

    # Test OERP.get
    loader = unittest.TestLoader().loadTestsFromTestCase(TestOSV)
    suite.addTest(loader)

    # Test socket timeout
    loader = unittest.TestLoader().loadTestsFromTestCase(TestTimeout)
    suite.addTest(loader)

    # Test session management
    loader = unittest.TestLoader().loadTestsFromTestCase(TestSession)
    suite.addTest(loader)

    # Test inspect service
    loader = unittest.TestLoader().loadTestsFromTestCase(TestInspect)
    suite.addTest(loader)

    #---------------
    #- Final Tests -
    #---------------

    # Test OERP.db (drop the database)
    if ARGS.create_db and ARGS.drop_db:
        loader = unittest.TestLoader().loadTestsFromTestCase(TestDBDrop)
        suite.addTest(loader)
    else:
        print("-- TestDBDrop skipped --")

    if (ARGS.log_file):
        log_file = ARGS.log_file
        with open(log_file, "w") as f:
            runTest(suite, f)
    else:
        runTest(suite, None)
       

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:



# -*- coding: UTF-8 -*-
##############################################################################
#
#    OERPLib
#    Copyright (C) 2011-2013 Sébastien Alix.
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published
#    by the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU Lesser General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
"""The `service` package provides classes to request services offers by
the `OpenERP/Odoo` server, such as:
    - '/common' (service.common),
    - '/db' (service.db)
    - '/wizard' (service.wizard)
    - Model, through '/object' (service.osv),
"""

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:



# -*- coding: UTF-8 -*-

try:
    import unittest2 as unittest
except:
    import unittest

from args import ARGS

import oerplib
from oerplib.service import osv


class TestLogin(unittest.TestCase):

    def test_oerp_no_db_login_db(self):
        # OERP no database + login database
        oerp = oerplib.OERP(
            ARGS.server, protocol=ARGS.protocol, port=ARGS.port,
            version=ARGS.version)
        user = oerp.login(ARGS.user, ARGS.passwd, ARGS.database)
        self.assertIsNotNone(user)
        self.assertIsInstance(user, osv.BrowseRecord)
        self.assertEqual(oerp.user, user)
        self.assertEqual(oerp.database, ARGS.database)

    def test_oerp_no_db_login_no_db(self):
        # OERP no database + login no database => Error
        oerp = oerplib.OERP(
            ARGS.server, protocol=ARGS.protocol, port=ARGS.port,
            version=ARGS.version)
        self.assertRaises(
            oerplib.error.Error,
            oerp.login, ARGS.user, ARGS.passwd)

    def test_oerp_db_login_no_db(self):
        # OERP database + login no database
        oerp = oerplib.OERP(
            ARGS.server, ARGS.database, protocol=ARGS.protocol, port=ARGS.port,
            version=ARGS.version)
        user = oerp.login(ARGS.user, ARGS.passwd)
        self.assertIsNotNone(user)
        self.assertIsInstance(user, osv.BrowseRecord)
        self.assertEqual(oerp.user, user)

    def test_oerp_db_login_db(self):
        # OERP database + login database
        oerp = oerplib.OERP(
            ARGS.server, ARGS.database, protocol=ARGS.protocol, port=ARGS.port,
            version=ARGS.version)
        user = oerp.login(ARGS.user, ARGS.passwd, ARGS.database)
        self.assertIsNotNone(user)
        self.assertIsInstance(user, osv.BrowseRecord)
        self.assertEqual(oerp.user, user)
        self.assertEqual(oerp.database, ARGS.database)

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:



# -*- coding: utf-8 -*-
##############################################################################
#
#    OERPLib
#    Copyright (C) 2013 Sébastien Alix.
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published
#    by the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU Lesser General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
"""Provide the :class:`Inspect` class which can output useful server data.

oerplib.service.inspect.Inspect
'''''''''''''''''''''''''''''''
"""
from functools import wraps

from oerplib import error


class Inspect(object):
    """.. versionadded:: 0.8

    The `Inspect` class provides methods to output useful server data.

    .. note::
        This service have to be used through the :attr:`oerplib.OERP.inspect`
        property.

    >>> import oerplib
    >>> oerp = oerplib.OERP('localhost')
    >>> user = oerp.login('admin', 'passwd', 'database')
    >>> oerp.inspect
    <oerplib.service.inspect.Inspect object at 0xb42fa84f>

    .. automethod:: relations(models, maxdepth=1, whitelist=['*'], blacklist=[], attrs_whitelist=[], attrs_blacklist=[], config={})

        Return a :class:`Relations <oerplib.service.inspect.relations.Relations>`
        object containing relations between data models, starting from `models`
        (depth = 0) and iterate recursively until reaching the `maxdepth` limit.

        `whitelist` and `blacklist` of models can be defined with patterns
        (a joker ``*`` can be used to match several models like ``account*``).
        The whitelist has a lower priority than the blacklist, and all models
        declared in `models` are automatically integrated to the `whitelist`.

        In the same way, displaying attributes can be defined for each model
        with ``attrs_whitelist`` and ``attrs_blacklist``. By default, model
        attributes are not displayed, unless the ``'*'`` pattern is supplied in
        ``attrs_whitelist``, or if only the ``attrs_blacklist`` is defined.

            >>> oerp.inspect.relations(
            ...     ['res.users'],
            ...     maxdepth=1,
            ...     whitelist=['res*'],
            ...     blacklist=['res.country*'],
            ...     attrs_whitelist=['*'],
            ...     attrs_blacklist=['res.partner', 'res.company'],
            ... ).write('res_users.png', format='png')

        `config` is a dictionary of options to override some attributes of
        the graph. Here the list of options and their default values:

            - ``relation_types: ['many2one', 'one2many', 'many2many']``,
            - ``show_many2many_table: False``,
            - ``color_many2one: #0E2548``,
            - ``color_one2many: #008200``,
            - ``color_many2many: #6E0004``,
            - ``model_root_bgcolor_title: #A50018``,
            - ``model_bgcolor_title: #64629C``,
            - ``model_color_title: white``,
            - ``model_color_subtitle': #3E3D60``,
            - ``model_bgcolor: white``,
            - ``color_normal: black``,
            - ``color_required: blue``
            - ``color_function: #D9602E``
            - ``space_between_models: 0.25``,

        >>> oerp.inspect.relations(
        ...     ['res.users'],
        ...     config={'relation_types': ['many2one']},  # Only show many2one relations
        ... ).write('res_users.png', format='png')

        .. note::
            With `OpenERP` < `6.1`, `many2one` and `one2many` relationships can
            not be bound together. Hence, a `one2many` relationship based on a
            `many2one` will draw a separate arrow.


    .. automethod:: dependencies(modules=[], models=[], models_blacklist=[], restrict=False, config={})

        Return a :class:`Dependencies <oerplib.service.inspect.dependencies.Dependencies>`
        object describing dependencies between modules. The `modules` defines
        a list of root nodes to reach among all dependencies (modules not
        related to them are not displayed). The default behaviour is to compute
        all dependencies between installed modules. The `models` list can be
        used to display all matching models among computed dependencies.

        `models` and `models_blacklist` parameters can be defined with patterns
        (a joker ``*`` can be used to match several models like ``account*``).
        The whitelist (`models`) has a lower priority than the blacklist
        (`models_blacklist`)::

            >>> oerp.inspect.dependencies(
            ...     models=['res.partner*'],
            ...     models_blacklist=['res.partner.title', 'res.partner.bank'],
            ... ).write('dependencies_res_partner.png', format='png')

        By default all installed modules are shown on the graph. To limit the
        result to modules related to the `base` one (its childs)::

            >>> oerp.inspect.dependencies(
            ...     ['base'],
            ...     ['res.partner*'],
            ...     ['res.partner.title', 'res.partner.bank'],
            ... ).write('dependencies_res_partner_base.png', format='png')

        All modules related to `base` are shown on the resulting graph, and
        matching models are highlighted among them, but some modules remain
        empty.
        To hide these "noisy" modules and restrict the resulting graph to
        data models that interest you, add the ``restrict=True`` parameter::

            >>> oerp.inspect.dependencies(
            ...     ['base'],
            ...     ['res.partner*'],
            ...     ['res.partner.title', 'res.partner.bank'],
            ...     restrict=True,
            ... ).write('dependencies_res_partner_base_restricted.png', format='png')

        In any case, root `modules` are always displayed on the graph in
        restricted mode (even if they have no matching model), and some
        unrelated modules may be added to satisfy dependencies.

        `config` is a dictionary of options to override some attributes of
        the graph. Here the list of options and their default values:

            - ``module_uninst_bgcolor_title: #DEDFDE``,
            - ``module_uninst_color_title: black``,
            - ``module_inst_bgcolor_title: #64629C``,
            - ``module_inst_color_title: white``,
            - ``module_root_bgcolor_title: #A50018``,
            - ``module_root_color_title: white``,
            - ``module_highlight_bgcolor_title: #1F931F``,
            - ``module_highlight_color_title: white``,
            - ``module_bgcolor: white``,
            - ``module_color_comment: grey``,
            - ``model_color_normal: black``,
            - ``model_color_transient: #7D7D7D``,
            - ``show_module_inst: True``,
            - ``show_module_uninst: False``,
            - ``show_model_normal: True``,
            - ``show_model_transient: False``,

        >>> oerp.inspect.dependencies(
        ...     ['base'],
        ...     ['res.partner*'],
        ...     ['res.partner.title', 'res.partner.bank'],
        ...     config={'show_model_transient': True},  # Show TransientModel/osv_memory models
        ... ).write('dependencies_res_partner_transient.png', format='png')

        .. note::
            With `OpenERP` `5.0`, data models can not be bound to their related
            modules, and as such the `models` and `models_blacklist`
            parameters are ignored.

    """
    def __init__(self, oerp):
        self._oerp = oerp

    def relations(self, models, maxdepth=1, whitelist=None, blacklist=None,
                  attrs_whitelist=None, attrs_blacklist=None, config=None):
        from oerplib.service.inspect.relations import Relations
        return Relations(
            self._oerp, models, maxdepth, whitelist, blacklist,
            attrs_whitelist, attrs_blacklist, config)

    def scan_on_change(self, models):
        """Scan all `on_change` methods detected among views of `models`, and
        returns a dictionary formatted as
        ``{model: {on_change: {view_id: field: [args]}}}``

            >>> oerp.inspect.scan_on_change(['sale.order'])
            {'sale.order': {
                'onchange_partner_id': {
                    'sale.view_order_form': {
                        'partner_id': ['partner_id']}},
                'onchange_partner_order_id': {
                    'sale.view_order_form': {
                        'partner_order_id': ['partner_order_id', 'partner_invoice_id', 'partner_shipping_id']}},
                'onchange_pricelist_id': {
                    'sale.view_order_form': {
                        'pricelist_id': ['pricelist_id', 'order_line']}},
                'onchange_shop_id': {
                    'sale.view_order_form': {
                        'shop_id': ['shop_id']}},
                'shipping_policy_change': {
                    'sale.view_order_form': {
                        'order_policy': ['order_policy']}}},
             'sale.order.line': {
                'product_id_change': {
                    'sale.view_order_form': {
                        'product_id': [
                            'parent.pricelist_id', 'product_id', 'product_uom_qty', 'product_uom',
                            'product_uos_qty', 'product_uos', 'name', 'parent.partner_id', False, True,
                            'parent.date_order', 'product_packaging', 'parent.fiscal_position', False, 'context'],
                        'product_uom_qty': [
                            'parent.pricelist_id', 'product_id', 'product_uom_qty', 'product_uom',
                            'product_uos_qty', 'product_uos', 'name', 'parent.partner_id', False, False,
                            'parent.date_order', 'product_packaging', 'parent.fiscal_position', True, 'context']}},
                ...
             }}
        """
        from oerplib.service.inspect.on_change import scan_on_change
        return scan_on_change(self._oerp, models)

    def dependencies(self, modules=None, models=None, models_blacklist=None,
                     restrict=False, config=None):
        from oerplib.service.inspect.dependencies import Dependencies
        return Dependencies(
            self._oerp, modules, models, models_blacklist, restrict, config)

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:



# -*- coding: utf-8 -*-
##############################################################################
#
#    OERPLib
#    Copyright (C) 2013 Sébastien Alix.
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published
#    by the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU Lesser General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
"""Provides the :class:`Proxy` and :class:`ProxyLegacy` classes."""
try:
    from urllib2 import build_opener, HTTPCookieProcessor, Request
except ImportError:
    from urllib.request import build_opener, HTTPCookieProcessor, Request

try:
    from cookielib import CookieJar
except ImportError:
    from http.cookiejar import CookieJar
import json
import random


class Proxy(object):
    """The :class:`Proxy` class provides a dynamic access
    to all JSON methods.
    """
    def __init__(self, host, port, timeout=120, ssl=False, deserialize=True):
        self._root_url = "{http}{host}:{port}".format(
            http=(ssl and "https://" or "http://"), host=host, port=port)
        self._timeout = timeout
        self._deserialize = deserialize
        self._builder = URLBuilder(self)
        cookie_jar = CookieJar()
        self._opener = build_opener(
            HTTPCookieProcessor(cookie_jar))

    def __getattr__(self, name):
        return getattr(self._builder, name)

    def __getitem__(self, url):
        return self._builder[url]

    def __call__(self, url, params):
        data = json.dumps({
            "jsonrpc": "2.0",
            "method": "call",
            "params": params,
            "id": random.randint(0, 1000000000),
        })
        request = Request(url='/'.join([self._root_url, url]))
        request.add_header('Content-Type', 'application/json')
        request.add_data(data)
        response = self._opener.open(request, timeout=self._timeout)
        if not self._deserialize:
            return response
        return json.load(response)


class ProxyLegacy(Proxy):
    """The :class:`ProxyLegacy` class fixes the request handling for
    OpenERP 6.1 and 7.0 by adding automatically the ``session_id`` parameter
    once the user is authenticated.
    """
    def __init__(self, host, port, timeout=120, ssl=False, deserialize=True):
        super(ProxyLegacy, self).__init__(host, port, timeout, ssl, deserialize)
        self._session_id = None

    def __call__(self, url, params):
        """Overloads the :func:`Proxy.__call__` method to add the 'session_id'
        parameter if necessary.
        """
        if url == 'web/session/authenticate':
            response = super(ProxyLegacy, self).__call__(url, params)
            result = self._deserialize and response or json.load(response)
            self._session_id = result and result['result']['session_id']
            return response
        elif self._session_id and 'session_id' not in params:
            params['session_id'] = self._session_id
        return super(ProxyLegacy, self).__call__(url, params)


class URLBuilder(object):
    """Auto-builds an URL while getting its attributes.
    Used by :class:`Proxy` and :class:`ProxyLegacy` classes.
    """
    def __init__(self, rpc, url=None):
        self._rpc = rpc
        self._url = url

    def __getattr__(self, path):
        new_url = self._url and '/'.join([self._url, path]) or path
        return URLBuilder(self._rpc, new_url)

    def __getitem__(self, path):
        if path and path[0] == '/':
            path = path[1:]
        if path and path[-1] == '/':
            path = path[:-1]
        return getattr(self, path)

    def __call__(self, **kwargs):
        return self._rpc(self._url, kwargs)

    def __str__(self):
        return self._url
# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:



# -*- coding: UTF-8 -*-
##############################################################################
#
#    OERPLib
#    Copyright (C) 2013 Sébastien Alix.
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published
#    by the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU Lesser General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
"""This module contains some helper functions used to save and load sessions
in `OERPLib`.
"""
import os
import stat
try:
    from ConfigParser import SafeConfigParser
except ImportError:
    from configparser import SafeConfigParser

from oerplib import error


def get_all(rc_file='~/.oerplibrc'):
    """Return all session configurations from the `rc_file` file.
    
    >>> import oerplib
    >>> oerplib.tools.session.get_all()
    {'foo': {'protocol': 'xmlrpc', 'user': 'admin', 'timeout': 120, 'database': 'db_name', 'passwd': 'admin', 'type': 'OERP', 'port': 8069, 'server': 'localhost'}}
    """
    conf = SafeConfigParser()
    conf.read([os.path.expanduser(rc_file)])
    sessions = {}
    for name in conf.sections():
        sessions[name] = {
            'type': conf.get(name, 'type'),
            'server': conf.get(name, 'server'),
            'protocol': conf.get(name, 'protocol'),
            'port': conf.getint(name, 'port'),
            'timeout': conf.getint(name, 'timeout'),
            'user': conf.get(name, 'user'),
            'passwd': conf.get(name, 'passwd'),
            'database': conf.get(name, 'database'),
        }
    return sessions


def get(name, rc_file='~/.oerplibrc'):
    """Return the session configuration identified by `name`
    from the `rc_file` file.

    >>> import oerplib
    >>> oerplib.tools.session.get('foo')
    {'protocol': 'xmlrpc', 'user': 'admin', 'timeout': 120, 'database': 'db_name', 'passwd': 'admin', 'type': 'OERP', 'port': 8069, 'server': 'localhost'}

    :raise: :class:`oerplib.error.Error`
    """
    conf = SafeConfigParser()
    conf.read([os.path.expanduser(rc_file)])
    if not conf.has_section(name):
        raise error.Error(
            "'{0}' session does not exist".format(name))
    return {
        'type': conf.get(name, 'type'),
        'server': conf.get(name, 'server'),
        'protocol': conf.get(name, 'protocol'),
        'port': conf.getint(name, 'port'),
        'timeout': conf.getint(name, 'timeout'),
        'user': conf.get(name, 'user'),
        'passwd': conf.get(name, 'passwd'),
        'database': conf.get(name, 'database'),
    }


#def list(rc_file='~/.oerplibrc'):
#    """Return a list of all sessions available in the
#    `rc_file` file.
#    """
#    conf = SafeConfigParser()
#    conf.read([os.path.expanduser(rc_file)])
#    # TODO
#    return conf.sections()


def save(name, data, rc_file='~/.oerplibrc'):
    """Save the `data` session configuration under the name `name`
    in the `rc_file` file.

    >>> import oerplib
    >>> oerplib.tools.session.save('foo', {'type': 'OERP', 'server': 'localhost', 'protocol': 'xmlrpc', 'port': 8069, 'timeout': 120, 'user': 'admin', 'passwd': 'admin', 'database': 'db_name'})
    """
    conf = SafeConfigParser()
    conf.read([os.path.expanduser(rc_file)])
    if not conf.has_section(name):
        conf.add_section(name)
    for k, v in data.iteritems():
        conf.set(name, k, str(v))
    with open(os.path.expanduser(rc_file), 'wb') as file_:
        os.chmod(os.path.expanduser(rc_file), stat.S_IREAD | stat.S_IWRITE)
        conf.write(file_)


def remove(name, rc_file='~/.oerplibrc'):
    """Remove the session configuration identified by `name`
    from the `rc_file` file.

    >>> import oerplib
    >>> oerplib.tools.session.remove('foo')

    :raise: :class:`oerplib.error.Error`
    """
    conf = SafeConfigParser()
    conf.read([os.path.expanduser(rc_file)])
    if not conf.has_section(name):
        raise error.Error(
            "'{0}' session does not exist".format(name))
    conf.remove_section(name)
    with open(os.path.expanduser(rc_file), 'wb') as file_:
        conf.write(file_)

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:



# -*- coding: UTF-8 -*-

try:
    import unittest2 as unittest
except:
    import unittest

from args import ARGS

import oerplib
from oerplib.tools import v


class TestDBCreate(unittest.TestCase):

    def setUp(self):
        self.oerp = oerplib.OERP(
            ARGS.server, protocol=ARGS.protocol, port=ARGS.port,
            version=ARGS.version)

    def test_db_create(self):
        if ARGS.database not in self.oerp.db.list():
            if v(self.oerp.version) >= v('8.0'):
                res = self.oerp.db.create_database(
                    ARGS.super_admin_passwd,
                    ARGS.database,
                    False,
                    'en_US',
                    ARGS.passwd)
                self.assertTrue(res)
            else:
                res = self.oerp.db.create_and_wait(
                    ARGS.super_admin_passwd,
                    ARGS.database,
                    False,
                    'en_US',
                    ARGS.passwd)
                self.assertIsInstance(res, list)
                self.assertNotEqual(res, list())
                self.assertEqual(res[0]['login'], 'admin')
                self.assertEqual(res[0]['password'], ARGS.passwd)
            import time
            time.sleep(10)

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:



# -*- coding: UTF-8 -*-
##############################################################################
#
#    OERPLib
#    Copyright (C) 2013 Sébastien Alix.
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published
#    by the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU Lesser General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
"""This module contains the :class:`Config <oerplib.config.Config>` class which
manage the configuration related to an instance of :class:`OERP <oerplib.OERP>`,
and some useful helper functions used internally in `OERPLib`.
"""
import collections
import re

MATCH_VERSION = re.compile(r'[^\d.]')


class Config(collections.MutableMapping):
    """Class which manage the configuration of an
    :class:`OERP <oerplib.OERP>` instance.

    .. note::
        This class have to be used through the :attr:`oerplib.OERP.config`
        property.

    >>> import oerplib
    >>> oerp = oerplib.OERP('localhost')
    >>> type(oerp.config)
    <class 'oerplib.tools.Config'>
    """
    def __init__(self, oerp, options):
        super(Config, self).__init__()
        self._oerp = oerp
        self._options = options or {}

    def __getitem__(self, key):
        return self._options[key]

    def __setitem__(self, key, value):
        """Handle ``timeout`` option to set the timeout on the connector."""
        if key == 'timeout':
            self._oerp._connector.timeout = value
        self._options[key] = value

    def __delitem__(self, key):
        # TODO raise exception
        pass

    def __iter__(self):
        return self._options.__iter__()

    def __len__(self):
        return len(self._options)

    def __str__(self):
        return self._options.__str__()

    def __repr__(self):
        return self._options.__repr__()


def clean_version(version):
    """Clean a version string.

        >>> from oerplib.tools import clean_version
        >>> clean_version('7.0alpha-20121206-000102')
        '7.0'

    :return: a cleaner version string
    """
    version = MATCH_VERSION.sub('', version.split('-')[0])
    return version


def detect_version(server, protocol, port, timeout=120):
    """
    .. deprecated:: 0.8

    Try to detect the server version.

        >>> from oerplib.tools import detect_version
        >>> detect_version('localhost', 'xmlrpc', 8069)
        '7.0'

    :return: the version as string
    """
    from oerplib import rpc
    # Try to request the server with the last API supported
    try:
        con = rpc.PROTOCOLS[protocol](
            server, port, protocol, timeout, version=None)
        version = con.db.server_version()
    except:
        # Try with the API of server < 6.1
        try:
            con = rpc.PROTOCOLS[protocol](
                server, port, protocol, timeout, version='6.0')
            version = con.db.server_version()
        except:
            # No version detected? Use the magic number in order to ensure the
            # use of the last API supported
            version = '42'
    finally:
        return clean_version(version)


def v(version):
    """Convert a version string to a tuple. The tuple can be use to compare
    versions between them.

        >>> from oerplib.tools import v
        >>> v('7.0')
        [7, 0]
        >>> v('6.1')
        [6, 1]
        >>> v('7.0') < v('6.1')
        False

    :return: the version as tuple
    """
    return [int(x) for x in clean_version(version).split(".")]

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:



# -*- coding: UTF-8 -*-

try:
    import unittest2 as unittest
except:
    import unittest

from args import ARGS

import oerplib


class TestOSV(unittest.TestCase):

    def setUp(self):
        self.oerp = oerplib.OERP(
            ARGS.server, protocol=ARGS.protocol, port=ARGS.port,
            version=ARGS.version)
        self.user = self.oerp.login(ARGS.user, ARGS.passwd, ARGS.database)

    def test_model(self):
        # Check the result returned
        self.oerp.get('res.users')

    def test_model_method(self):
        # Check the result returned
        model = self.oerp.get('res.users')
        model.name_get(self.user.id)
        self.oerp.get('ir.sequence').get('fake.code')  # Return False

    def test_model_method_without_args(self):
        # Handle exception (execute a 'name_get' with without args)
        model = self.oerp.get('res.users')
        self.assertRaises(
            oerplib.error.RPCError,
            model.name_get)

    def test_model_method_with_wrong_args(self):
        # Handle exception (execute a 'search' with wrong args)
        model = self.oerp.get('res.users')
        self.assertRaises(
            oerplib.error.RPCError,
            model.search,
            False)  # Wrong arg

    def test_model_browse_with_one_id(self):
        # Check the result returned
        model = self.oerp.get('res.users')
        user = model.browse(self.user.id)
        self.assertEqual(user, self.user)

    def test_model_browse_with_ids(self):
        # Iteration
        for result in self.oerp.get('res.users').browse([self.user.id]):
            self.assertEqual(self.user, result)
        user_ids = self.oerp.search('res.users', [])
        for result in self.oerp.get('res.users').browse(user_ids):
            self.assertIsInstance(
                result, oerplib.service.osv.browse.BrowseRecord)
        # With context
        context = self.oerp.execute('res.users', 'context_get')
        for result in self.oerp.get('res.users').browse(user_ids, context):
            self.assertIsInstance(
                result, oerplib.service.osv.browse.BrowseRecord)


# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:



# -*- coding: UTF-8 -*-
##############################################################################
#
#    OERPLib
#    Copyright (C) 2011-2013 Sébastien Alix.
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published
#    by the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU Lesser General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
"""This module contains the ``OERP`` class which is the entry point to manage 
an `OpenERP/Odoo` server.
"""
import os
import base64
import zlib
import tempfile
import time

from oerplib import rpc, error, tools
from oerplib.tools import session, v
from oerplib.service import common, db, wizard, osv, inspect

try:
   long
except NameError:
   long = int

class OERP(object):
    """Return a new instance of the :class:`OERP` class.
    The optional `database` parameter specifies the default database to use
    when the :func:`login <oerplib.OERP.login>` method is called.
    If no `database` is set, the `database` parameter of the
    :func:`login <oerplib.OERP.login>` method will be mandatory.

    `XML-RPC` and `Net-RPC` protocols are supported. Respective values for the
    `protocol` parameter are ``xmlrpc``, ``xmlrpc+ssl`` and ``netrpc``.

        >>> import oerplib
        >>> oerp = oerplib.OERP('localhost', protocol='xmlrpc', port=8069)

    Since the version `0.7`, `OERPLib` will try by default to detect the
    server version in order to adapt its requests. However, it is
    possible to force the version to use with the `version` parameter:

        >>> oerp = oerplib.OERP('localhost', version='6.0')

    :raise: :class:`oerplib.error.InternalError`,
        :class:`oerplib.error.RPCError`
    """

    def __init__(self, server='localhost', database=None, protocol='xmlrpc',
                 port=8069, timeout=120, version=None):
        if protocol not in ['xmlrpc', 'xmlrpc+ssl', 'netrpc']:
            txt = ("The protocol '{0}' is not supported by the OERP class. "
                   "Please choose a protocol among these ones: {1}")
            txt = txt.format(protocol, ['xmlrpc', 'xmlrpc+ssl', 'netrpc'])
            raise error.InternalError(txt)
        self._server = server
        self._port = port
        self._protocol = protocol
        self._database = self._database_default = database
        self._uid = None
        self._password = None
        self._user = None
        self._common = common.Common(self)
        self._db = db.DB(self)
        self._wizard = wizard.Wizard(self)
        self._inspect = inspect.Inspect(self)
        # Instanciate the server connector
        try:
            self._connector = rpc.PROTOCOLS[protocol](
                self._server, self._port, timeout, version)
        except rpc.error.ConnectorError as exc:
            raise error.InternalError(exc.message)
        # Dictionary of configuration options
        self._config = tools.Config(
            self,
            {'auto_context': True,
             'timeout': timeout})

    @property
    def config(self):
        """Dictionary of available configuration options.

        >>> oerp.config
        {'auto_context': True, 'timeout': 120}

        - ``auto_context``: if set to `True`, the user context will be sent
          automatically to every call of a
          :class:`model <oerplib.service.osv.Model>` method (default: `True`):

            .. versionadded:: 0.7

            .. note::

                This option only works on servers in version `6.1` and above.

            >>> product_osv = oerp.get('product.product')
            >>> product_osv.name_get([3]) # Context sent by default ('lang': 'fr_FR' here)
            [[3, '[PC1] PC Basic']]
            >>> oerp.config['auto_context'] = False
            >>> product_osv.name_get([3]) # No context sent
            [[3, '[PC1] Basic PC']]

        - ``timeout``: set the maximum timeout in seconds for a RPC request
          (default: `120`):

            .. versionadded:: 0.6

            >>> oerp.config['timeout'] = 300

        """
        return self._config

    # Readonly properties
    @property
    def user(self):
        """The browsable record of the user connected.

        >>> oerp.login('admin', 'admin') == oerp.user
        True

        """
        return self._user

    @property
    def context(self):
        """The context of the user connected.

        >>> oerp.login('admin', 'admin')
        browse_record('res.users', 1)
        >>> oerp.context
        {'lang': 'fr_FR', 'tz': False}
        >>> oerp.context['lang'] = 'en_US'

        """
        return self._context

    @property
    def version(self):
        """The version of the server.

        >>> oerp.version
        '7.0-20131014-231047'
        """
        return self._connector.version

    server = property(lambda self: self._server,
                      doc="The server name.")
    port = property(lambda self: self._port,
                    doc="The port used.")
    protocol = property(lambda self: self._protocol,
                        doc="The protocol used.")

    database = property(lambda self: self._database,
                        doc="The database currently used.")
    common = property(lambda self: self._common,
                      doc=(""".. versionadded:: 0.6

                       The common service (``/common`` RPC service).
                       See the :class:`oerplib.service.common.Common` class."""))
    db = property(lambda self: self._db,
                  doc=(""".. versionadded:: 0.4

                       The database management service (``/db`` RPC service).
                       See the :class:`oerplib.service.db.DB` class."""))
    wizard = property(lambda self: self._wizard,
                      doc=(""".. versionadded:: 0.6

                       The wizard service (``/wizard`` RPC service).
                       See the :class:`oerplib.service.wizard.Wizard` class."""))

    inspect = property(lambda self: self._inspect,
                       doc=(""".. versionadded:: 0.8

                       The inspect service (custom service).
                       See the :class:`oerplib.service.inspect.Inspect`
                       class."""))

    #NOTE: in the past this function was implemented as a decorator for other
    # methods needed to be checked, but Sphinx documentation generator is not
    # able to auto-document decorated methods.
    def _check_logged_user(self):
        """Check if a user is logged. Otherwise, an error is raised."""
        if not self._uid or not self._password:
            raise error.Error(u"User login required.")

    def login(self, user='admin', passwd='admin', database=None):
        """Log in as the given `user` with the password `passwd` on the
        database `database` and return the corresponding user as a browsable
        record (from the ``res.users`` model).
        If `database` is not specified, the default one will be used instead.

        >>> user = oerp.login('admin', 'admin', database='db_name')
        >>> user.name
        u'Administrator'

        :return: the user connected as a browsable record
        :raise: :class:`oerplib.error.RPCError`, :class:`oerplib.error.Error`
        """
        # Raise an error if no database was given
        self._database = database or self._database_default
        if not self._database:
            raise error.Error("No database specified")
        # Get the user's ID and generate the corresponding User record
        try:
            user_id = self.common.login(self._database, user, passwd)
        except rpc.error.ConnectorError as exc:
            raise error.RPCError(exc.message, exc.oerp_traceback)
        else:
            if user_id:
                self._uid = user_id
                self._password = passwd
                self._context = self.execute('res.users', 'context_get')
                self._user = self.browse('res.users', user_id, self._context)
                return self._user
            else:
                #FIXME: Raise an error?
                raise error.RPCError("Wrong login ID or password")

    # ------------------------- #
    # -- Raw XML-RPC methods -- #
    # ------------------------- #

    def execute(self, model, method, *args):
        """Execute the `method` of `model`.
        `*args` parameters varies according to the `method` used.

        >>> oerp.execute('res.partner', 'read', [1, 2], ['name'])
        [{'name': u'ASUStek', 'id': 2}, {'name': u'Your Company', 'id': 1}]

        :return: the result returned by the `method` called
        :raise: :class:`oerplib.error.RPCError`
        """
        self._check_logged_user()
        # Execute the query
        try:
            return self._connector.object.execute(
                self._database, self._uid,
                self._password,
                model, method, *args)
        except rpc.error.ConnectorError as exc:
            raise error.RPCError(exc.message, exc.oerp_traceback)

    def execute_kw(self, model, method, args=None, kwargs=None):
        """Execute the `method` of `model`.
        `args` is a list of parameters (in the right order),
        and `kwargs` a dictionary (named parameters). Both varies according
        to the `method` used.

        >>> oerp.execute_kw('res.partner', 'read', [[1, 2]], {'fields': ['name']})
        [{'name': u'ASUStek', 'id': 2}, {'name': u'Your Company', 'id': 1}]

        .. warning::

            This method only works on servers in version `6.1` and above.

        :return: the result returned by the `method` called
        :raise: :class:`oerplib.error.RPCError`
        """
        self._check_logged_user()
        # Execute the query
        args = args or []
        kwargs = kwargs or {}
        try:
            return self._connector.object.execute_kw(
                self._database, self._uid, self._password,
                model, method, args, kwargs)
        except rpc.error.ConnectorError as exc:
            raise error.RPCError(exc.message, exc.oerp_traceback)

    def exec_workflow(self, model, signal, obj_id):
        """Execute the workflow `signal` on
        the instance having the ID `obj_id` of `model`.

        :raise: :class:`oerplib.error.RPCError`
        """
        #TODO NEED TEST
        self._check_logged_user()
        # Execute the workflow query
        try:
            self._connector.object.exec_workflow(
                self._database, self._uid, self._password,
                model, signal, obj_id)
        except rpc.error.ConnectorError as exc:
            raise error.RPCError(exc.message, exc.oerp_traceback)

    def report(self, report_name, model, obj_ids, report_type='pdf',
               context=None):
        """Download a report from the server and return the local
        path of the file.

        >>> oerp.report('sale.order', 'sale.order', 1)
        '/tmp/oerplib_uJ8Iho.pdf'
        >>> oerp.report('sale.order', 'sale.order', [1, 2])
        '/tmp/oerplib_giZS0v.pdf'

        :return: the path to the generated temporary file
        :raise: :class:`oerplib.error.RPCError`
        """
        #TODO report_type: what it means exactly?

        self._check_logged_user()
        # If no context was supplied, get the default one
        if context is None:
            context = self.context
        # Execute the report query
        try:
            if v(self.version) < v('6.1'):
                pdf_data = self._get_report_data_v5(
                    report_name, model, obj_ids, report_type, context)
            else:
                pdf_data = self._get_report_data_v61(
                    report_name, model, obj_ids, report_type, context)
        except rpc.error.ConnectorError as exc:
            raise error.RPCError(exc.message, exc.oerp_traceback)
        return self._print_file_data(pdf_data)

    def _get_report_data_v61(self, report_name, model, obj_ids,
                             report_type='pdf', context=None):
        """Download binary data of a report from the server.
        It uses the `render_report` RPC method available since OpenERP 6.1.
        """
        if context is None:
            context = {}
        obj_ids = [obj_ids] if isinstance(obj_ids, (int, long)) else obj_ids
        data = {
            'model': model,
            'id': obj_ids[0],
            'ids': obj_ids,
            'report_type': report_type,
        }
        try:
            return self._connector.report.render_report(
                self._database, self.user.id, self._password,
                report_name, obj_ids, data, context)
        except rpc.error.ConnectorError as exc:
            raise error.RPCError(exc.message, exc.oerp_traceback)

    def _get_report_data_v5(self, report_name, model, obj_ids,
                            report_type='pdf', context=None):
        """Download binary data of a report from the server."""
        context = context or {}
        obj_ids = [obj_ids] if isinstance(obj_ids, (int, long)) else obj_ids
        data = {
            'model': model,
            'id': obj_ids[0],
            'ids': obj_ids,
            'report_type': report_type,
        }
        try:
            report_id = self._connector.report.report(
                self._database, self.user.id, self._password,
                report_name, obj_ids, data, context)
        except rpc.error.ConnectorError as exc:
            raise error.RPCError(exc.message, exc.oerp_traceback)
        state = False
        attempt = 0
        while not state:
            try:
                pdf_data = self._connector.report.report_get(
                    self._database, self.user.id, self._password,
                    report_id)
            except rpc.error.ConnectorError as exc:
                raise error.RPCError("Unknown error occurred during the "
                                     "download of the report.")
            state = pdf_data['state']
            if not state:
                time.sleep(1)
                attempt += 1
            if attempt > 200:
                raise error.RPCError("Download time exceeded, "
                                     "the operation has been canceled.")
        return pdf_data

    @staticmethod
    def _print_file_data(data):
        """Print data in a temporary file and return the path of this one."""
        if 'result' not in data:
            raise error.InternalError(
                "Invalid data, the operation has been canceled.")
        content = base64.decodestring(data['result'])
        if data.get('code') == 'zlib':
            content = zlib.decompress(content)

        if data['format'] in ['pdf', 'html', 'doc', 'xls',
                              'sxw', 'odt', 'tiff']:
            if data['format'] == 'html' and os.name == 'nt':
                data['format'] = 'doc'
            (file_no, file_path) = tempfile.mkstemp('.' + data['format'],
                                                    'oerplib_')
            with file(file_path, 'wb+') as fp:
                fp.write(content)
            os.close(file_no)
            return file_path

    # ------------------------- #
    # -- High Level methods  -- #
    # ------------------------- #

    def browse(self, model, ids, context=None):
        """Browse one or several records (if `ids` is a list of IDs)
        from `model`. The fields and values for such objects are generated
        dynamically.

        >>> oerp.browse('res.partner', 1)
        browse_record(res.partner, 1)

        >>> [partner.name for partner in oerp.browse('res.partner', [1, 2])]
        [u'Your Company', u'ASUStek']

        A list of data types used by ``browse_record`` fields are
        available :ref:`here <fields>`.

        :return: a ``browse_record`` instance
        :return: a generator to iterate on ``browse_record`` instances
        :raise: :class:`oerplib.error.RPCError`
        """
        return self.get(model).browse(ids, context)

    def search(self, model, args=None, offset=0, limit=None, order=None,
               context=None, count=False):
        """Return a list of IDs of records matching the given criteria in
        `args` parameter. `args` must be of the form
        ``[('name', '=', 'John'), (...)]``

        >>> oerp.search('res.partner', [('name', 'like', 'Agrolait')])
        [3]

        :return: a list of IDs
        :raise: :class:`oerplib.error.RPCError`
        """
        if args is None:
            args = []
        context = context or self._context
        return self.execute(model, 'search', args, offset, limit, order,
                            context, count)

    def create(self, model, vals, context=None):
        """Create a new `model` record with values contained in the `vals`
        dictionary.

        >>> partner_id = oerp.create('res.partner', {'name': 'Jacky Bob', 'lang': 'fr_FR'})

        :return: the ID of the new record.
        :raise: :class:`oerplib.error.RPCError`
        """
        context = context or self._context
        return self.execute(model, 'create', vals, context)

    def read(self, model, ids, fields=None, context=None):
        """Return `fields` values for each `model` record identified by `ids`.
        If `fields` is not specified, all fields values will be retrieved.

        >>> oerp.read('res.partner', [1, 2], ['name'])
        [{'name': u'ASUStek', 'id': 2}, {'name': u'Your Company', 'id': 1}]

        :return: list of dictionaries
        :raise: :class:`oerplib.error.RPCError`
        """
        if fields is None:
            fields = []
        context = context or self._context
        return self.execute(model, 'read', ids, fields, context)

    def write(self, model, ids, vals=None, context=None):
        """Update `model` records identified by `ids` with the given values
        contained in the `vals` dictionary.

        >>> oerp.write('res.users', [1], {'name': "Administrator"})
        True

        :return: `True`
        :raise: :class:`oerplib.error.RPCError`
        """
        #if ids is None:
        #    ids = []
        if vals is None:
            vals = {}
        context = context or self._context
        return self.execute(model, 'write', ids, vals, context)

    def unlink(self, model, ids, context=None):
        """Delete `model` records identified by `ids`.

        >>> oerp.unlink('res.partner', [1])

        :return: `True`
        :raise: :class:`oerplib.error.RPCError`
        """
        #if ids is None:
        #    ids = []
        context = context or self._context
        return self.execute(model, 'unlink', ids, context)

    # ---------------------- #
    # -- Special methods  -- #
    # ---------------------- #

    def write_record(self, browse_record, context=None):
        """.. versionadded:: 0.4

        Update the record corresponding to `browse_record` by sending its values
        to the server (only field values which have been changed).

        >>> partner = oerp.browse('res.partner', 1)
        >>> partner.name = "Test"
        >>> oerp.write_record(partner)  # write('res.partner', [1], {'name': "Test"})

        :return: `True`
        :raise: :class:`oerplib.error.RPCError`
        """
        if not isinstance(browse_record, osv.BrowseRecord):
            raise ValueError("An instance of BrowseRecord is required")
        return osv.Model(self, browse_record.__osv__['name'])._write_record(
            browse_record, context)

    def unlink_record(self, browse_record, context=None):
        """.. versionadded:: 0.4

        Delete the record corresponding to `browse_record` from the server.

        >>> partner = oerp.browse('res.partner', 1)
        >>> oerp.unlink_record(partner)  # unlink('res.partner', [1])

        :return: `True`
        :raise: :class:`oerplib.error.RPCError`
        """
        if not isinstance(browse_record, osv.BrowseRecord):
            raise ValueError("An instance of BrowseRecord is required")
        return osv.Model(self, browse_record.__osv__['name'])._unlink_record(
            browse_record, context)

    def refresh(self, browse_record, context=None):
        """Restore original values on `browse_record` with data
        fetched on the server.
        As a result, all changes made locally on the record are canceled.

        :raise: :class:`oerplib.error.RPCError`
        """
        return osv.Model(self, browse_record.__osv__['name'])._refresh(
            browse_record, context)

    def reset(self, browse_record):
        """Cancel all changes made locally on the `browse_record`.
        No request to the server is executed to perform this operation.
        Therefore, values restored may be outdated.
        """
        return osv.Model(self, browse_record.__osv__['name'])._reset(
            browse_record)

    @staticmethod
    def get_osv_name(browse_record):
        """
        .. deprecated:: 0.7 use the ``__osv__`` attribute instead
           (see :class:`BrowseRecord <oerplib.service.osv.BrowseRecord>`).

        >>> partner = oerp.browse('res.partner', 1)
        >>> oerp.get_osv_name(partner)
        'res.partner'

        :return: the model name of the browsable record

        """
        if not isinstance(browse_record, osv.BrowseRecord):
            raise ValueError("Value is not a browse_record.")
        return browse_record.__osv__['name']

    def get(self, model):
        """.. versionadded:: 0.5

        Return a proxy of the `model` built from the
        server (see :class:`oerplib.service.osv.Model`).

        :return: an instance of :class:`oerplib.service.osv.Model`
        """
        return osv.Model(self, model)

    def save(self, name, rc_file='~/.oerplibrc'):
        """.. versionadded:: 0.8

        Save the session configuration under the name `name`.
        These informations are stored in the ``~/.oerplibrc`` file by default.

            >>> import oerplib
            >>> oerp = oerplib.OERP('localhost', protocol='xmlrpc', port=8069)
            >>> oerp.login('admin', 'admin', 'db_name')
            >>> oerp.save('foo')

        Such informations can be loaded with the :func:`oerplib.load` function
        by returning a pre-configured session of :class:`OERP <oerplib.OERP>`,
        or with the `oerp` command line tool supplied with `oerplib`.
        """
        self._check_logged_user()
        data = {
            'type': self.__class__.__name__,
            'server': self.server,
            'protocol': self.protocol,
            'port': self.port,
            'timeout': self.config['timeout'],
            'user': self.user.login,
            'passwd': self._password,
            'database': self.database,
        }
        session.save(name, data, rc_file)

    @classmethod
    def load(cls, name, rc_file='~/.oerplibrc'):
        """.. versionadded:: 0.8

        Return a :class:`OERP` session pre-configured and connected
        with informations identified by `name`:

            >>> import oerplib
            >>> oerp = oerplib.OERP.load('foo')

        Such informations are stored with the :func:`OERP.save <oerplib.OERP.save>`
        method.
        """
        data = session.get(name, rc_file)
        if data.get('type') != cls.__name__:
            raise error.Error(
                "'{0}' session is not of type '{1}'".format(
                    name, cls.__name__))
        oerp = cls(
            server=data['server'],
            protocol=data['protocol'],
            port=data['port'],
            timeout=data['timeout'],
        )
        oerp.login(
            user=data['user'], passwd=data['passwd'],
            database=data['database'])
        return oerp

    @classmethod
    def list(cls, rc_file='~/.oerplibrc'):
        """.. versionadded:: 0.8

        Return a list of all sessions available in the
        `rc_file` file:

            >>> import oerplib
            >>> oerplib.OERP.list()
            ['foo', 'bar']

        Then, use the :func:`load` function with the desired session:

            >>> oerp = oerplib.OERP.load('foo')
        """
        sessions = session.get_all(rc_file)
        return [name for name, data in sessions.iteritems()
                if data.get('type') == cls.__name__]
        #return session.list(rc_file)

    @classmethod
    def remove(cls, name, rc_file='~/.oerplibrc'):
        """.. versionadded:: 0.8

        Remove the session identified by `name` from the `rc_file` file:

            >>> import oerplib
            >>> oerplib.OERP.remove('foo')
            True
        """
        data = session.get(name, rc_file)
        if data.get('type') != cls.__name__:
            raise error.Error(
                "'{0}' session is not of type '{1}'".format(
                    name, cls.__name__))
        return session.remove(name, rc_file)

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:



# -*- coding: UTF-8 -*-

import argparse

_parser = argparse.ArgumentParser(description="Run tests for OERPLib.")

_parser.add_argument('--server', default='localhost',
                     help="Host")
#_parser.add_argument('--protocol', default='xmlrpc',
#                     help="Protocol")
#_parser.add_argument('--port', default='8069',
#                     help="Port")

_parser.add_argument('--test_xmlrpc', action='store_true',
                     help="Test the XML-RPC protocol")
_parser.add_argument('--xmlrpc_port', default='8069',
                     help="Port to use with the XML-RPC protocol")
_parser.add_argument('--test_netrpc', action='store_true',
                     help="Test the NET-RPC protocol")
_parser.add_argument('--netrpc_port', default='8070',
                     help="Port to use with the NET-RPC protocol")
_parser.add_argument('--version', default=None,
                     help="OpenERP version used")

_parser.add_argument('--super_admin_passwd', default='admin',
                     help="Super administrator password")

_parser.add_argument('--database', default='oerplib_test',
                     help="Name of the database")

_parser.add_argument('--user', default='admin',
                     help="User login")
_parser.add_argument('--passwd', default='admin',
                     help="User password")

_parser.add_argument('--create_db', action='store_true',
                     help="Create a database for tests")
_parser.add_argument('--drop_db', action='store_true',
                     help="Drop the created database. "
                          "Works only with --create_db")

_parser.add_argument('--verbosity', default=2, type=int,
                     help="Output verbosity of unittest")

_parser.add_argument('--log_file', default=None,
                     help="Where to write the log")

ARGS = _parser.parse_args()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:



# -*- coding: UTF-8 -*-
##############################################################################
#
#    OERPLib
#    Copyright (C) 2012-2013 Sébastien Alix.
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published
#    by the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU Lesser General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

try:
    import xmlrpclib
except ImportError:
    from xmlrpc import client as xmlrpclib

try:
    import httplib
except ImportError:
    from http import client as httplib
import socket
import sys

try:
    from urlparse import urlparse
except ImportError:
    from urllib.parse import urlparse

# Defined later following the version of Python used
TimeoutTransport = None  
TimeoutSafeTransport = None


class TimeoutServerProxy(xmlrpclib.ServerProxy):
    """xmlrpclib.ServerProxy overload to manage the timeout of the socket."""
    def __init__(self, *args, **kwargs):
        url = args[0]
        https_ok = urlparse(url).scheme == 'https'
        t = https_ok and TimeoutSafeTransport() or TimeoutTransport()
        t.timeout = kwargs.get('timeout', 120)
        if 'timeout' in kwargs:
            del kwargs['timeout']
        kwargs['transport'] = t
        xmlrpclib.ServerProxy.__init__(self, *args, **kwargs)


if sys.version_info <= (2, 7):
    # Python 2.5 and 2.6

    # -- xmlrpclib.Transport with timeout support --

    class TimeoutHTTPPy26(httplib.HTTP):
        def __init__(self, host='', port=None, strict=None,
                     timeout=socket._GLOBAL_DEFAULT_TIMEOUT):
            if port == 0:
                port = None
            self._setup(self._connection_class(host, port, strict, timeout))

    class TimeoutTransportPy26(xmlrpclib.Transport):
        def __init__(self, timeout=socket._GLOBAL_DEFAULT_TIMEOUT,
                     *args, **kwargs):
            xmlrpclib.Transport.__init__(self, *args, **kwargs)
            self.timeout = timeout

        def make_connection(self, host):
            host, extra_headers, x509 = self.get_host_info(host)
            conn = TimeoutHTTPPy26(host, timeout=self.timeout)
            return conn

    # -- xmlrpclib.SafeTransport with timeout support --

    class TimeoutHTTPSPy26(httplib.HTTPS):
        def __init__(self, host='', port=None, key_file=None, cert_file=None,
                     strict=None, timeout=socket._GLOBAL_DEFAULT_TIMEOUT):
            if port == 0:
                port = None
            self._setup(self._connection_class(
                host, port, key_file, cert_file, strict, timeout))
            self.key_file = key_file
            self.cert_file = cert_file

    class TimeoutSafeTransportPy26(xmlrpclib.SafeTransport):
        def __init__(self, timeout=socket._GLOBAL_DEFAULT_TIMEOUT,
                     *args, **kwargs):
            xmlrpclib.Transport.__init__(self, *args, **kwargs)
            self.timeout = timeout

        def make_connection(self, host):
            host, extra_headers, x509 = self.get_host_info(host)
            conn = TimeoutHTTPSPy26(host, timeout=self.timeout)
            return conn

    # Define the TimeTransport and TimeSafeTransport class version to use
    TimeoutTransport = TimeoutTransportPy26
    TimeoutSafeTransport = TimeoutSafeTransportPy26
else:
    # Python 2.7 and 3.X

    # -- xmlrpclib.Transport with timeout support --

    class TimeoutHTTPConnectionPy27(httplib.HTTPConnection):
        def __init__(self, timeout, *args, **kwargs):
            httplib.HTTPConnection.__init__(self, *args, **kwargs)
            self.timeout = timeout

        def connect(self):
            httplib.HTTPConnection.connect(self)
            self.sock.settimeout(self.timeout)

    class TimeoutTransportPy27(xmlrpclib.Transport):
        def __init__(self, timeout=socket._GLOBAL_DEFAULT_TIMEOUT,
                     *args, **kwargs):
            xmlrpclib.Transport.__init__(self, *args, **kwargs)
            self.timeout = timeout

        def make_connection(self, host):
            if self._connection and host == self._connection[0]:
                return self._connection[1]

            chost, self._extra_headers, x509 = self.get_host_info(host)
            self._connection = host, TimeoutHTTPConnectionPy27(
                self.timeout, chost)
            return self._connection[1]

    # -- xmlrpclib.SafeTransport with timeout support --

    class TimeoutHTTPSConnectionPy27(httplib.HTTPSConnection):
        def __init__(self, timeout, *args, **kwargs):
            httplib.HTTPSConnection.__init__(self, *args, **kwargs)
            self.timeout = timeout

        def connect(self):
            httplib.HTTPSConnection.connect(self)
            self.sock.settimeout(self.timeout)

    class TimeoutSafeTransportPy27(xmlrpclib.SafeTransport):
        def __init__(self, timeout=socket._GLOBAL_DEFAULT_TIMEOUT,
                     *args, **kwargs):
            xmlrpclib.SafeTransport.__init__(self, *args, **kwargs)
            self.timeout = timeout

        def make_connection(self, host):
            if self._connection and host == self._connection[0]:
                return self._connection[1]

            chost, self._extra_headers, x509 = self.get_host_info(host)
            self._connection = host, TimeoutHTTPSConnectionPy27(
                self.timeout, chost)
            return self._connection[1]

    # Define the TimeTransport and TimeSafeTransport class version to use
    TimeoutTransport = TimeoutTransportPy27
    TimeoutSafeTransport = TimeoutSafeTransportPy27

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:



#!/usr/bin/env python
"""A sample script to demonstrate some of functionalities of OERPLib."""
import oerplib

# XMLRPC server configuration (NETRPC is also supported)
SERVER = 'localhost'
PROTOCOL = 'xmlrpc'
PORT = 8069
# Name of the OpenERP database to use
DATABASE = 'db_name'

USER = 'admin'
PASSWORD = 'password'

try:
    # Login
    oerp = oerplib.OERP(
        server=SERVER, database=DATABASE, protocol=PROTOCOL, port=PORT)
    oerp.login(USER, PASSWORD)

    # ----------------------- #
    # -- Low level methods -- #
    # ----------------------- #

    # Execute - search
    user_ids = oerp.execute('res.users', 'search', [('id', '=', oerp.user.id)])
    # Execute - read
    user_data = oerp.execute('res.users', 'read', user_ids[0])
    # Execute - write
    oerp.execute('res.users', 'write', user_ids[0], {'name': "Administrator"})
    # Execute - create
    new_user_id = oerp.execute('res.users', 'create', {'login': "New user"})

    # --------------------- #
    # -- Dynamic methods -- #
    # --------------------- #

    # Get the model
    user_obj = oerp.get('res.users')
    # Search IDs of a model that match criteria
    user_obj.search([('name', 'ilike', "Administrator")])
    # Create a record
    new_user_id = user_obj.create({'login': "new_user"})
    # Read data of a record (just the name field)
    user_data = user_obj.read([new_user_id], ['name'])
    # Write a record
    user_obj.write([new_user_id], {'name': "New user"})
    # Delete a record
    user_obj.unlink([new_user_id])

    # -------------------- #
    # -- Browse objects -- #
    # -------------------- #

    # Browse an object
    user = user_obj.browse(oerp.user.id)
    print(user.name)
    print(user.company_id.name)
    # .. or many objects
    order_obj = oerp.get('sale.order')
    for order in order_obj.browse([68, 69]):
        print(order.name)
        print(order.partner_id.name)
        for line in order.order_line:
            print('\t{0}'.format(line.name))

    # ----------------------- #
    # -- Download a report -- #
    # ----------------------- #

    so_pdf_path = oerp.report('sale.order', 'sale.order', 1)
    inv_pdf_path = oerp.report('webkitaccount.invoice', 'account.invoice', 1)

    # ------------------------- #
    # -- Databases management-- #
    # ------------------------- #

    # List databases
    print(oerp.db.list())
    # Create a database in background
    oerp.db.create(
        'super_admin_passwd', 'my_db',
        demo_data=True, lang='fr_FR',
        admin_passwd='admin_passwd')
    # ... after a while, dump it
    my_dump = oerp.db.dump('super_admin_password', 'my_db')
    # Create a new database from the dump
    oerp.db.restore('super_admin_password', 'my_new_db', my_dump)
    # Delete the old one
    oerp.db.drop('super_admin_password', 'my_db')

except oerplib.error.Error as e:
    print(e)
except Exception as e:
    print(e)



# -*- coding: UTF-8 -*-
##############################################################################
#
#    OERPLib
#    Copyright (C) 2011-2013 Sébastien Alix.
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published
#    by the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU Lesser General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
"""Provide the :class:`Common` class to manage the /common RPC service."""

from oerplib import rpc, error


class Common(object):
    """.. versionadded:: 0.6

    The `Common` class represents the ``/common`` RPC service which lets you
    log in on the server, and provides various utility functions.

    .. note::
        This service have to be used through the :attr:`oerplib.OERP.common`
        property.

    >>> import oerplib
    >>> oerp = oerplib.OERP('localhost')
    >>> oerp.common
    <oerplib.service.common.Common object at 0xb76266ac>

    .. warning::

        All methods documented below are not strictly implemented in `OERPLib`

        Method calls are purely dynamic, and the following documentation can be
        wrong if the API of the server is changed between versions. Anyway,
        if you know the API proposed by the server for the ``/common`` RPC
        service, it will work.

    .. method:: Common.login(db, login, password)

        >>> oerp.common.login('test_db', 'admin', 'admin_passwd')
        1

        :return: the user's ID or `False`

    .. method:: Common.authenticate(db, login, password, user_agent_env)

        >>> oerp.common.authenticate('test_db', 'admin', 'admin_passwd', {})
        1

        :return: the user's ID or `False`

    .. method:: Common.version()

        >>> oerp.common.version()
        {'protocol_version': 1, 'server_version': '6.1'}

    .. method:: Common.about(extended=False)

        Return information about the server.

        >>> oerp.common.about()
        'See http://openerp.com'

        >>> oerp.common.about(True)
        ['See http://openerp.com', '8.0alpha1']

        :param: extended: if `True` then return version info
        :return: string if extended is `False` else tuple

    .. method:: Common.timezone_get(db, login, password)

        >>> oerp.common.timezone_get('test_db', 'admin', 'admin_passwd')
        'UTC'

    .. method:: Common.get_server_environment()

        >>> print(oerp.common.get_server_environment())
        Environment Information :
        System : Linux-2.6.32-5-686-i686-with-debian-6.0.4
        OS Name : posix
        Distributor ID:	Debian
        Description:	Debian GNU/Linux 6.0.4 (squeeze)
        Release:	6.0.4
        Codename:	squeeze
        Operating System Release : 2.6.32-5-686
        Operating System Version : #1 SMP Mon Mar 26 05:20:33 UTC 2012
        Operating System Architecture : 32bit
        Operating System Locale : fr_FR.UTF8
        Python Version : 2.6.6
        OpenERP-Server Version : 5.0.16
        Last revision No. & ID :

    .. method:: Common.login_message()

        >>> oerp.common.login_message()
        'Welcome'

    .. method:: Common.set_loglevel(loglevel, logger=None)

        >>> oerp.common.set_loglevel('DEBUG')

    .. method:: Common.get_stats()

        >>> print(oerp.common.get_stats())
        OpenERP server: 5 threads
        Servers started
        Net-RPC: running

    .. method:: Common.list_http_services()

        >>> oerp.common.list_http_services()
        []

    .. method:: Common.check_connectivity()

        >>> oerp.common.check_connectivity()
        True

    .. method:: Common.get_os_time()

        >>> oerp.common.get_os_time()
        (0.01, 0.0, 0.0, 0.0, 17873633.129999999)

    .. method:: Common.get_sqlcount()

        >>> oerp.common.get_sqlcount()

    .. method:: Common.get_available_updates(super_admin_password, contract_id, contract_password)

        >>> oerp.common.get_available_updates('super_admin_passwd', 'MY_CONTRACT_ID', 'MY_CONTRACT_PASSWORD')

    .. method:: Common.get_migration_scripts(super_admin_password, contract_id, contract_password)

        >>> oerp.common.get_migration_scripts('super_admin_passwd', 'MY_CONTRACT_ID', 'MY_CONTRACT_PASSWORD')

    """
    def __init__(self, oerp):
        self._oerp = oerp

    def __getattr__(self, method):
        """Provide a dynamic access to a RPC method."""
        def rpc_method(*args):
            """Return the result of the RPC request."""
            try:
                meth = getattr(self._oerp._connector.common, method, False)
                return meth(*args)
            except rpc.error.ConnectorError as exc:
                raise error.RPCError(exc.message, exc.oerp_traceback)
        return rpc_method

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:



# -*- coding: UTF-8 -*-

try:
    import unittest2 as unittest
except:
    import unittest
import socket

from args import ARGS

import oerplib


class TestTimeout(unittest.TestCase):

    def setUp(self):
        self.oerp = oerplib.OERP(
            ARGS.server, protocol=ARGS.protocol, port=ARGS.port,
            version=ARGS.version)
        self.user = self.oerp.login(ARGS.user, ARGS.passwd, ARGS.database)

    def test_reduced_timeout(self):
        ids = self.oerp.search('ir.module.module', [])
        # Set the timeout
        self.oerp.config['timeout'] = 0.1
        # Execute a time consuming query: handle exception
        self.assertRaises(
            socket.timeout,
            self.oerp.write, 'ir.module.module', ids, {})

    def test_increased_timeout(self):
        # Set the timeout
        self.oerp.config['timeout'] = 120
        # Execute a time consuming query: no exception
        ids = self.oerp.search('ir.module.module', [])
        self.oerp.write('ir.module.module', ids, {})

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:



# -*- coding: UTF-8 -*-

try:
    import unittest2 as unittest
except:
    import unittest

from args import ARGS

import oerplib


class TestInspect(unittest.TestCase):

    def setUp(self):
        self.oerp = oerplib.OERP(
            ARGS.server, protocol=ARGS.protocol, port=ARGS.port,
            version=ARGS.version)
        self.user = self.oerp.login(ARGS.user, ARGS.passwd, ARGS.database)

    def test_relations(self):
        graph = self.oerp.inspect.relations(
            ['res.users'],
            maxdepth=1,
            whitelist=['*'], blacklist=[],
            attrs_whitelist=['*'], attrs_blacklist=[])
        self.assertIsInstance(graph, oerplib.service.inspect.relations.Relations)

    def test_relations_maxdepth_null(self):
        graph = self.oerp.inspect.relations(['res.users'], maxdepth=0)
        self.assertIsInstance(graph, oerplib.service.inspect.relations.Relations)

    def test_relations_maxdepth_negative(self):
        graph = self.oerp.inspect.relations(['res.users'], maxdepth=-1)
        self.assertIsInstance(graph, oerplib.service.inspect.relations.Relations)

    def test_dependencies(self):
        graph = self.oerp.inspect.dependencies()
        self.assertIsInstance(
            graph, oerplib.service.inspect.dependencies.Dependencies)

    def test_dependencies_with_module(self):
        graph = self.oerp.inspect.dependencies(['base'])
        self.assertIsInstance(
            graph, oerplib.service.inspect.dependencies.Dependencies)

    def test_dependencies_with_models(self):
        graph = self.oerp.inspect.dependencies(models=['res.partner*'])
        self.assertIsInstance(
            graph, oerplib.service.inspect.dependencies.Dependencies)

    def test_dependencies_with_wrong_module(self):
        self.assertRaises(
            oerplib.error.InternalError,
            self.oerp.inspect.dependencies, ['wrong_module'])

    def test_dependencies_with_module_and_models(self):
        graph = self.oerp.inspect.dependencies(['base'], ['res.partner*'])
        self.assertIsInstance(
            graph, oerplib.service.inspect.dependencies.Dependencies)

    def test_dependencies_with_module_and_models_restricted(self):
        graph = self.oerp.inspect.dependencies(
            ['base'], ['res.partner*'], restrict=True)
        self.assertIsInstance(
            graph, oerplib.service.inspect.dependencies.Dependencies)

    def test_dependencies_with_module_and_models_blacklist(self):
        graph = self.oerp.inspect.dependencies(
            ['base'], ['res.partner*'], ['res.partner.bank'])
        self.assertIsInstance(
            graph, oerplib.service.inspect.dependencies.Dependencies)

    def test_dependencies_with_module_and_models_blacklist_restricted(self):
        graph = self.oerp.inspect.dependencies(
            ['base'], ['res.partner*'], ['res.partner.bank'], restrict=True)
        self.assertIsInstance(
            graph, oerplib.service.inspect.dependencies.Dependencies)

    def test_scan_on_change(self):
        res = self.oerp.inspect.scan_on_change(['res.users'])
        self.assertIsInstance(res, dict)
        res = self.oerp.inspect.scan_on_change(['res.users', 'res.partner'])
        self.assertIsInstance(res, dict)

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:



# -*- coding: UTF-8 -*-
##############################################################################
#
#    OERPLib
#    Copyright (C) 2011-2013 Sébastien Alix.
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published
#    by the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU Lesser General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
"""Provide the :class:`DB` class in order to manage the server databases."""

import time

from oerplib import rpc, error


class DB(object):
    """.. versionadded:: 0.4

    The `DB` class represents the database management service.
    It provides functionalities such as list, create, drop, dump
    and restore databases.

    .. note::
        This service have to be used through the :attr:`oerplib.OERP.db`
        property.

    >>> import oerplib
    >>> oerp = oerplib.OERP('localhost')
    >>> oerp.db
    <oerplib.service.db.DB object at 0xb75fb04c>

    .. warning::

        All methods documented below are not strictly implemented in `OERPLib`
        (except the
        :func:`create_and_wait <oerplib.service.db.DB.create_and_wait>` method).

        Method calls are purely dynamic, and the following documentation can be
        wrong if the API of the server is changed between versions. Anyway, if
        you know the API proposed by the server for the ``/db`` RPC
        service, it will work.

    .. method:: DB.list()

        Return a list of the databases:

        >>> oerp.db.list()
        ['prod_db', 'test_db']

        :return: a list of database names

    .. method:: DB.list_lang()

        Return a list of codes and names of language supported by the server:

        >>> oerp.db.list_lang()
        [['sq_AL', 'Albanian / Shqipëri'], ['ar_AR', 'Arabic / الْعَرَبيّة'], ...]

        :return: a list of pairs representing languages with their codes and
                 names

    .. method:: DB.server_version()

        Return the version of the server:

        >>> oerp.db.server_version()
        '6.1'

        :return: the version of the server as string

    .. method:: DB.dump(super_admin_passwd, database)

        Return a dump of `database` in `base64`:

        >>> binary_data = oerp.db.dump('super_admin_passwd', 'prod_db')

        The super administrator password `super_admin_passwd` is
        required to perform this action.

        :return: the `base64` string representation of the `database`

    .. method:: DB.restore(super_admin_passwd, database, binary_data)

        Restore in `database` a dump previously created with the
        :func:`dump <DB.dump>` method:

        >>> oerp.db.restore('super_admin_passwd', 'test_db', binary_data)

        The super administrator password `super_admin_passwd` is
        required to perform this action.

    .. method:: DB.drop(super_admin_passwd, database)

        Drop the `database`:

        >>> oerp.db.drop('super_admin_passwd', 'test_db')
        True

        The super administrator password `super_admin_passwd` is
        required to perform this action.

        :return: `True`

    .. method:: DB.create(super_admin_passwd, database, demo_data=False, lang='en_US', admin_passwd='admin')

        Request the server to create a new database named `database`
        which will have `admin_passwd` as administrator password and localized
        with the `lang` parameter.
        You have to set the flag `demo_data` to `True` in order to insert
        demonstration data.

        As the creating process may take some time, you can execute the
        :func:`get_progress <DB.get_progress>` method with the database ID
        returned to know its current state.

        >>> database_id = oerp.db.create('super_admin_passwd', 'test_db', False, 'fr_FR', 'my_admin_passwd')

        The super administrator password `super_admin_passwd` is
        required to perform this action.

        :return: the ID of the new database

    .. method:: DB.get_progress(super_admin_passwd, database_id)

        Check the state of the creating process for the database identified by
        the `database_id` parameter.

        >>> oerp.db.get_progress('super_admin_passwd', database_id) # Just after the call to the 'create' method
        (0, [])
        >>> oerp.db.get_progress('super_admin_passwd', database_id) # Once the database is fully created
        (1.0, [{'login': 'admin', 'password': 'admin', 'name': 'Administrator'},
               {'login': 'demo', 'password': 'demo', 'name': 'Demo User'}])

        The super administrator password `super_admin_passwd` is
        required to perform this action.

        :return: A tuple with the progressing state and a list
                of user accounts created (once the database is fully created).

    .. method:: DB.create_database(super_admin_passwd, database, demo_data=False, lang='en_US', admin_passwd='admin')

        `Available since OpenERP 6.1`

        Similar to :func:`create <DB.create>` but blocking.

        >>> oerp.db.create_database('super_admin_passwd', 'test_db', False, 'fr_FR', 'my_admin_passwd')
        True

        The super administrator password `super_admin_passwd` is
        required to perform this action.

        :return: `True`

    .. method:: DB.duplicate_database(super_admin_passwd, original_database, database)

        `Available since OpenERP 7.0`

        Duplicate `original_database' as `database`.

        >>> oerp.db.duplicate_database('super_admin_passwd', 'prod_db', 'test_db')
        True

        The super administrator password `super_admin_passwd` is
        required to perform this action.

        :return: `True`

    .. method:: DB.rename(super_admin_passwd, old_name, new_name)

        Rename the `old_name` database to `new_name`.

        >>> oerp.db.rename('super_admin_passwd', 'test_db', 'test_db2')
        True

        The super administrator password `super_admin_passwd` is
        required to perform this action.

        :return: `True`

    .. method:: DB.db_exist(database)

        Check if connection to database is possible.

        >>> oerp.db.db_exist('prod_db')
        True

        :return: `True` or `False`

    .. method:: DB.change_admin_password(super_admin_passwd, new_passwd)

        Change the administrator password by `new_passwd`.

        >>> oerp.db.change_admin_password('super_admin_passwd', 'new_passwd')
        True

        The super administrator password `super_admin_passwd` is
        required to perform this action.

        :return: `True`

    """
    def __init__(self, oerp):
        self._oerp = oerp

    def create_and_wait(self, super_admin_passwd, database, demo_data=False,
                        lang='en_US', admin_passwd='admin'):
        """
        .. note::

            This method is not part of the official API. It's just
            a wrapper around the :func:`create <DB.create>` and
            :func:`get_progress <DB.get_progress>` methods. For server
            in version `6.1` or above, please prefer the use of the
            standard :func:`create_database <DB.create_database>` method.

        Like the :func:`create <DB.create>` method, but waits the end of
        the creating process by executing the
        :func:`get_progress <DB.get_progress>` method regularly to check its
        state.

        >>> oerp.db.create_and_wait('super_admin_passwd', 'test_db', False, 'fr_FR', 'my_admin_passwd')
        [{'login': 'admin', 'password': 'my_admin_passwd', 'name': 'Administrateur'},
         {'login': 'demo', 'password': 'demo', 'name': 'Demo User'}]

        The super administrator password `super_admin_passwd` is
        required to perform this action.

        :return: a list of user accounts created
        :raise: :class:`oerplib.error.RPCError`

        """
        try:
            db_id = self._oerp._connector.db.create(
                super_admin_passwd, database, demo_data, lang, admin_passwd)
            progress = 0.0
            attempt = 0
            while progress < 1.0:
                result = self._oerp._connector.db.get_progress(
                    super_admin_passwd, db_id)
                progress = result[0]
                if progress < 1.0:
                    time.sleep(1)
                    attempt += 1
                if attempt > 300:
                    raise error.RPCError(
                        "Too many attempts, the operation"
                        " has been canceled.")
            return result[1]

        except rpc.error.ConnectorError as exc:
            #FIXME handle the exception with the UnicodeEncodeError for
            # the error 'the database already exists'.
            #print dir(exc)
            raise error.RPCError(exc.message, exc.oerp_traceback)

    def __getattr__(self, method):
        """Provide a dynamic access to a RPC method."""
        def rpc_method(*args):
            """Return the result of the RPC request."""
            try:
                meth = getattr(self._oerp._connector.db, method, False)
                return meth(*args)
            except rpc.error.ConnectorError as exc:
                raise error.RPCError(exc.message, exc.oerp_traceback)
        return rpc_method

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:



# -*- coding: UTF-8 -*-

try:
    import unittest2 as unittest
except:
    import unittest
import numbers
import time

from args import ARGS

import oerplib
from oerplib.tools import v


class TestExecuteKw(unittest.TestCase):

    def setUp(self):
        self.oerp = oerplib.OERP(
            ARGS.server, protocol=ARGS.protocol, port=ARGS.port,
            version=ARGS.version)
        if v(self.oerp.version) < v('6.1'):
            raise unittest.SkipTest(
                "The targetted OpenERP server does not support the "
                "'execute_kw()' method.")
        self.user = self.oerp.login(ARGS.user, ARGS.passwd, ARGS.database)

    # ------
    # Search
    # ------
    def test_execute_kw_search_with_good_args(self):
        # Check the result returned
        result = self.oerp.execute_kw('res.users', 'search', [[]], {})
        self.assertIsInstance(result, list)
        self.assertIn(self.user.id, result)
        result = self.oerp.execute_kw(
            'res.users', 'search',
            [[('id', '=', self.user.id)]], {'order': 'name'})
        self.assertIn(self.user.id, result)
        self.assertEqual(result[0], self.user.id)

    def test_execute_kw_search_without_args(self):
        # Handle exception (execute a 'search' without args)
        self.assertRaises(
            oerplib.error.RPCError,
            self.oerp.execute_kw,
            'res.users',
            'search')

    def test_execute_kw_search_with_wrong_args(self):
        # Handle exception (execute a 'search' with wrong args)
        self.assertRaises(
            oerplib.error.RPCError,
            self.oerp.execute_kw,
            'res.users',
            'search',
            False, False)   # Wrong args

    def test_execute_kw_search_with_wrong_model(self):
        # Handle exception (execute a 'search' with a wrong model)
        self.assertRaises(
            oerplib.error.RPCError,
            self.oerp.execute_kw,
            'wrong.model',  # Wrong model
            'search',
            [[]], {})

    def test_execute_kw_search_with_wrong_method(self):
        # Handle exception (execute a 'search' with a wrong method)
        self.assertRaises(
            oerplib.error.RPCError,
            self.oerp.execute_kw,
            'res.users',
            'wrong_method',  # Wrong method
            [[]], {})

    # ------
    # Create
    # ------
    def test_execute_kw_create_with_good_args(self):
        login = "%s_%s" % ("foobar", time.time())
        # Check the result returned
        result = self.oerp.execute_kw(
            'res.users', 'create',
            [{'name': login, 'login': login}])
        self.assertIsInstance(result, numbers.Number)
        # Handle exception (create another user with the same login)
        self.assertRaises(
            oerplib.error.RPCError,
            self.oerp.execute_kw,
            'res.users', 'create',
            [{'name': login, 'login': login}])

    def test_execute_kw_create_without_args(self):
        # Handle exception (execute a 'create' without args)
        self.assertRaises(
            oerplib.error.RPCError,
            self.oerp.execute_kw,
            'res.users',
            'create')

    def test_execute_kw_create_with_wrong_args(self):
        # Handle exception (execute a 'create' with wrong args)
        self.assertRaises(
            oerplib.error.RPCError,
            self.oerp.execute_kw,
            'res.users',
            'create',
            True, True)   # Wrong args

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
